DEFINT A-Z
'$DYNAMIC
'$INCLUDE: 'COMMON.BI'
'$INCLUDE: 'DEXTERN.BI'
'$INCLUDE: 'DIRECTQB.BI'

' User types
TYPE PlayerType   ' Player data type
	x AS SINGLE: y AS SINGLE
	xMove AS SINGLE: yMove AS SINGLE
	Walk AS INTEGER: Jump AS INTEGER
	Look AS INTEGER: Fall AS INTEGER
	Stat AS INTEGER: Dir AS INTEGER
	Shoot AS INTEGER: Death AS INTEGER
	Health AS INTEGER
END TYPE

TYPE EnemyType    ' Enemy data type
	x AS SINGLE: y AS SINGLE
	xMove AS SINGLE: yMove AS SINGLE
	Dir AS INTEGER: Found AS INTEGER
	EnType AS INTEGER: HitCount AS INTEGER
END TYPE

TYPE BulletType   ' Bullet data type
	x AS SINGLE: y AS SINGLE
	xMove AS SINGLE: yMove AS SINGLE
	Active AS INTEGER: BulType AS INTEGER
END TYPE

TYPE ParticleType ' Particle data type
	x AS SINGLE: y AS SINGLE
	xMove AS SINGLE: yMove AS SINGLE
	Active AS INTEGER: pType AS INTEGER
END TYPE

TYPE ItemType  ' Item data type
	x AS INTEGER
	y AS INTEGER
	iType AS INTEGER
END TYPE

TYPE TagType   ' Tag data type
	TagPos AS INTEGER
	TagDat AS INTEGER
END TYPE

TYPE CameraType   ' Camera data type
	x AS INTEGER
	y AS INTEGER
END TYPE

' Sub/function declarations
DECLARE SUB ActivateTags (HTag%)
DECLARE SUB Msgbox (Layer%, Msg$)
DECLARE SUB SetTile (x%, y%, Layer%, Tile%)
DECLARE SUB ScrollLayer (Layer%, xMove%, yMove%)
DECLARE SUB GETsprite (Index%, SpriteSeg%, SpriteOff%)
DECLARE SUB Particlize (x%, y%, SpriteSeg%, SpriteOff%, Col%)
DECLARE SUB PutParticle (x!, y!, xMove!, yMove!, pType%)
DECLARE SUB GetTagData (Tag%, xPos%, yPos%, HTag%, LTag%)
DECLARE SUB SetTagData (Tag%, xPos%, yPos%, HTag%, LTag%)
DECLARE FUNCTION GetWinMode% ()
DECLARE FUNCTION GetHitFlag% (Tile%)
DECLARE FUNCTION GetTile% (x%, y%, Layer%)
DECLARE FUNCTION GETmemRequired& (GETfile$)
DECLARE FUNCTION GETprocess% (GETfile$, TempLayer%, TempSeg%, TempOff%, TempPal$)

' Constants
CONST PI# = 3.141592653589793#

' Defined variables
COMMON Pal AS STRING * 768    ' Palette data
COMMON Camera AS CameraType   ' Camera coordinates type
COMMON Player AS PlayerType   ' Player data type
COMMON SHARED TagCount        ' Level tag count

ON ERROR GOTO ErrorHandler

' Make sure that a valid mode of windows is running
WinMode = GetWinMode
SELECT CASE WinMode
	CASE 0: PRINT "This program requires Microsoft Windows."; CHR$(13): END
	CASE 1, 2: PRINT "This program requires Microsoft Windows in 386 Enhanced mode."; CHR$(13): END
END SELECT

' Check for missing files
LvlFile$ = "LEVELS\CRAPLAND.SLV"
IF COMMAND$ <> "" THEN LvlFile$ = COMMAND$
IF FileNotFound(LvlFile$) THEN PRINT "I cannot load "; LCASE$(LvlFile$); "!": END
IF FileNotFound("IMAGES\SAKURA0.PCX") THEN PRINT "I cannot load images\sakura0.pcx!": END
IF FileNotFound("IMAGES\SPRITES.PCX") THEN PRINT "I cannot load images\sprites.pcx!": END
IF FileNotFound("SAKURA.GET") THEN PRINT "I cannot load sakura.get!": END
IF FileNotFound("SPRITES.GET") THEN PRINT "I cannot load sprites.get!": END
IF FileNotFound("ANIMEPAL.PAL") THEN PRINT "I cannot load animepal.pal!": END
IF FileNotFound("MICRO.FNT") THEN PRINT "I cannot load micro.fnt!": END

' Init and load stuff
PRINT "Choose a sound playback quality (use number keys):"
PRINT "1 - Low quality at 11KHz (Pentium 200MHz)"
PRINT "2 - Medium quality at 22KHz (Cyrix MII 333MHz)"
PRINT "3 - High quality at 44KHz (Pentium II 400MHz or faster)"

DO
	P$ = INKEY$
	IF P$ = "1" THEN Quality& = LOWQUALITY: EXIT DO
	IF P$ = "2" THEN Quality& = MEDIUMQUALITY: EXIT DO
	IF P$ = "3" THEN Quality& = HIGHQUALITY: EXIT DO
LOOP

CLS
RANDOMIZE TIMER
' Initialize sound engine
IF WinMode = 3 THEN OS = WIN9X ELSE OS = WINNT
OPEN "SOUNDSYS.CFG" FOR BINARY AS #1
PUT #1, , OS
CLOSE #1
PRINT "Initializing DS4QB++...";
SELECT CASE DS4QB.Init(Quality&, DEFAULT)
	CASE -1: PRINT "DS4QB++ critical system files in SOUNDSYS\ are missing!": END
	CASE -2: PRINT "DS4QB++ cannot communicate with slave module!": END
	CASE -3: PRINT "DS4QB++ configuration file (soundsys.dat) is missing!": END
	CASE ELSE: IF OS = WIN9X THEN PRINT "Initialized in Win9x mode." ELSE PRINT "Initialized in WinNT mode."
END SELECT

' Declare arrays once FAT DS4QB++ loads

' Data arrays
DIM Tile(580 \ 2)                   ' Tile data
DIM LevelData(7500)                 ' Level data
DIM Tags(200) AS LONG               ' Level tags
DIM TileFlag(255)                   ' Tile hit/draw flags
DIM TileAnim(255)                   ' Tile animation flags
DIM AnimRate(255)                   ' Tile animation rate
DIM AnimData(255)                   ' Animation system data
DIM TileFrame(255)                  ' Tile frame
DIM LineBuffer(160)                 ' Buffer for backdrop scrolling
DIM Particle(100) AS ParticleType   ' Particle data array
DIM Bullet(100) AS BulletType       ' Bullet data array
DIM Enemy(100) AS EnemyType         ' Enemy data array
DIM Item(100) AS ItemType           ' Item data type
DIM SCRN(32001) AS INTEGER          ' Screen buffer
DIM CT(360) AS SINGLE               ' Cosine table
DIM ST(360) AS SINGLE               ' Sine table

' Arrays and variables for EMS sprite manipulation system
DIM SHARED NewSpriteIndex                    ' Next sprite index to use
DIM SHARED SpriteDataOffs AS LONG            ' Sprite data offset
DIM SHARED SpriteSize(50)                    ' Sprite size array
DIM SHARED SpriteOffs(50) AS LONG            ' Sprite offset array
DIM SpriteBuff(DQBsize(1, 1, 128, 128) \ 2)  ' Sprite buffer for EMS transfer

' Initialize library
PRINT "Initializing DirectQB...";
SpriteDataOffs = 178480
MemRequired& = GETmemRequired("SAKURA.GET")
MemRequired& = MemRequired& + GETmemRequired("SPRITES.GET")
SELECT CASE DQBinit(2, 0, 190 + (MemRequired& \ 1024) + 1)
	CASE 1: PRINT "386 compatible processor not found!": DS4QB.Close: END
	CASE 2: PRINT "Cannot find an Expanded Memory Manager!": DS4QB.Close: END
	CASE 3: PRINT "Not enough free EMS memory to initialize!": DS4QB.Close: END
	CASE 4: PRINT "Library already initialized!": DS4QB.Close: END
END SELECT
DQBfpu
PRINT "Done."

' Load sounds
PRINT "Loading sounds...";
DS4QB.LoadSound 1, "SOUNDS\SHOOT.WAV", DEFAULT
DS4QB.LoadSound 2, "SOUNDS\RIC1.WAV", DEFAULT
DS4QB.LoadSound 3, "SOUNDS\RIC2.WAV", DEFAULT
DS4QB.LoadSound 4, "SOUNDS\RIC3.WAV", DEFAULT
DS4QB.LoadSound 5, "SOUNDS\SWITCH.WAV", DEFAULT
DS4QB.LoadSound 6, "SOUNDS\BLOWUP.WAV", DEFAULT
DS4QB.LoadSound 7, "SOUNDS\STUMPED.WAV", DEFAULT
DS4QB.LoadSound 8, "SOUNDS\ZIPGUNS.WAV", DEFAULT
DS4QB.LoadSound 9, "SOUNDS\BOUNCY.WAV", DEFAULT
DS4QB.LoadSound 10, "SOUNDS\WORMDIE!.WAV", DEFAULT
DS4QB.LoadSound 11, "SOUNDS\WORMHURT.WAV", DEFAULT
DS4QB.LoadSound 12, "SOUNDS\WORMDIED.WAV", DEFAULT
DS4QB.LoadSound 13, "SOUNDS\BULLSEYE.WAV", DEFAULT
DS4QB.LoadSound 14, "SOUNDS\DEATH.WAV", DEFAULT
DS4QB.LoadSound 15, "SOUNDS\WORMFIRE.WAV", DEFAULT
DS4QB.LoadSound 16, "SOUNDS\SLIME.WAV", DEFAULT
DS4QB.LoadSound 17, "SOUNDS\GETSTUFF.WAV", DEFAULT
DS4QB.LoadMusic 1, "NEATSONG.IT", MUS.LOOPING
PRINT "Done."

' Load sprites and data
PRINT "Loading sprites...";
Crap = GETprocess("SAKURA.GET", 1, VARSEG(SpriteBuff(0)), VARPTR(SpriteButt(0)), Pal)
Crap = GETprocess("SPRITES.GET", 1, VARSEG(SpriteBuff(0)), VARPTR(SpriteButt(0)), Pal)
Crap = DQBloadFont("MICRO.FNT")
PRINT "Done."

' Generate math tables
PRINT "Generating math tables...";
FOR I = 0 TO 360
	CT(I) = COS(I * PI# / 180)
	ST(I) = SIN(I * PI# / 180)
NEXT I
PRINT "Done."

' Init graphics and load palette
DQBinitVGA
'$INCLUDE: 'LOADPAL.BI'
'$INCLUDE: 'COLORS.BI'
DQBgetPal Pal

' Level loading code
OPEN LvlFile$ FOR BINARY AS #1
IF INPUT$(4, 1) <> "SLV1" THEN   ' Check for ID marker
	CLOSE #1
	Msgbox VIDEO, LvlFile$ + "is not a valid Sakura Level file!"
	WHILE INKEY$ = "": WEND
	GOTO EndIt
END IF
' Read level parameters
MapXdim = ASC(INPUT$(1, 1))               ' Map X dimension
MapYdim = ASC(INPUT$(1, 1))               ' Map Y dimension
ScrollMode = ASC(INPUT$(1, 1))            ' Backdrop scroll mode
ScrollRate = ASC(INPUT$(1, 1))            ' Backdrop scroll rate
SpriteCount = ASC(INPUT$(1, 1))           ' Sprite count
TagCount = ASC(INPUT$(1, 1))              ' Tag count
Backdrop$ = INPUT$(ASC(INPUT$(1, 1)), 1)  ' Backdrop image file
SetFile$ = INPUT$(ASC(INPUT$(1, 1)), 1)   ' Tileset file
' Load backdrop image
IF DQBloadImage(2, 0, 0, Backdrop$, Pal, 0, 0) THEN
	Msgbox VIDEO, "I cannot load " + Backdrop$ + "!"
	WHILE INKEY$ = "": WEND
	GOTO EndIt
END IF
' Load level data
FOR Layer = 0 TO 1
	FOR Ly = 1 TO MapYdim
		Buffer$ = INPUT$(MapXdim, 1)
		FOR Lx = 1 TO MapXdim
			Offs = (Lx - 1) + 150 * (Ly - 1)
			Num = ASC(MID$(Buffer$, Lx, 1))
			DEF SEG = VARSEG(LevelData(0))
			POKE VARPTR(LevelData(0)) + Offs, Num
			DEF SEG
			' Put your level loading animation here
		NEXT Lx
	NEXT Ly
	' Copy level layer to EMS
	DQBpoke VARSEG(LevelData(0)), VARPTR(LevelData(0)), 148480 + (15000 * Layer), 15000
NEXT Layer
FOR r = 1 TO SpriteCount
	xPos = CVI(INPUT$(2, 1))
	yPos = CVI(INPUT$(2, 1))
	Index = CVI(INPUT$(2, 1))
	' Put your starting position detection here
	SELECT CASE Index
		CASE 1
			pStartX = xPos
			pStartY = yPos
		CASE 2
			EnemyCount = EnemyCount + 1
			Enemy(EnemyCount).x = xPos
			Enemy(EnemyCount).y = yPos
			Enemy(EnemyCount).EnType = 1
		CASE 3
			EnemyCount = EnemyCount + 1
			Enemy(EnemyCount).x = xPos
			Enemy(EnemyCount).y = yPos
			Enemy(EnemyCount).Dir = 1
			Enemy(EnemyCount).EnType = 3
		CASE 4
			ItemCount = ItemCount + 1
			Item(ItemCount).x = xPos
			Item(ItemCount).y = yPos
			Item(ItemCount).iType = 1
		CASE 5
			ItemCount = ItemCount + 1
			Item(ItemCount).x = xPos
			Item(ItemCount).y = yPos
			Item(ItemCount).iType = 2
	END SELECT
NEXT r
' Read tag data then close the file
FOR r = 1 TO TagCount: GET #1, , Tags(r): NEXT
CLOSE #1
' Tileset loading code
OPEN SetFile$ FOR BINARY AS #1
IF LOF(1) = 0 THEN
	CLOSE #1: KILL SetFile$
	Msgbox VIDEO, "I cannot load " + SetFile$ + "!"
	WHILE INKEY$ = "": WEND
	GOTO EndIt
END IF
IF INPUT$(4, 1) <> "STS1" THEN
	CLOSE #1
	Msgbox VIDEO, SetFile$ + " is not a valid Sakura Tileset file!"
	WHILE INKEY$ = "": WEND
	GOTO EndIt
END IF
' Load tile data
Tile(0) = 24 * 8
Tile(1) = 24
FOR Load = 0 TO 255
	TileAnim(Load) = CVI(INPUT$(2, 1))
	TileFlag(Load) = CVI(INPUT$(2, 1))
	AnimRate(Load) = ASC(INPUT$(1, 1))
	FOR Ry = 1 TO 24
		Buffer$ = INPUT$(24, 1)
		FOR Rx = 1 TO 24
			Pix = ASC(MID$(Buffer$, Rx, 1))
			Offs = (Rx - 1) + 24 * (Ry - 1)
			DEF SEG = VARSEG(Tile(2))
			POKE VARPTR(Tile(2)) + Offs, Pix
			DEF SEG
			' Put your tileset loading animation here
		NEXT Rx
	NEXT Ry
	' Copy tile to EMS
	DQBpoke VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(580) * Load, 580
NEXT Load
CLOSE #1
' ============================================================================

DS4QB.PlayMusic 1

' Reset vars
Player.x = pStartX
Player.y = pStartY
Player.Health = 100
HScrollCount = 0
VScrollCount = 0
vSync = 1

GOSUB FixCamera

' Main loop
DQBinstallKeyboard
DO

	' Scroll camera
	YOffs = 0
	IF Player.Stat = 0 THEN
		SELECT CASE Player.Look
			CASE -1: YOffs = 80
			CASE 1: YOffs = -80
		END SELECT
	END IF
	FOR Scroll = 1 TO 20
		' Up
		IF (Player.y + 31) - Camera.y < (100 + YOffs) - 10 * Scroll AND Camera.y > 0 THEN
			IF ScrollMode = 2 OR ScrollMode = 3 THEN VScrollCount = VScrollCount + 1
			IF VScrollCount > ScrollRate THEN
				ScrollLayer 2, 0, 1
				VScrollCount = 0
			END IF
			Camera.y = Camera.y - 1
		END IF
		' Down
		IF (Player.y + 31) - Camera.y > (100 + YOffs) + 10 * Scroll AND Camera.y < (24 * MapYdim) - 200 THEN
			IF ScrollMode = 2 OR ScrollMode = 3 THEN VScrollCount = VScrollCount + 1
			IF VScrollCount > ScrollRate THEN
				ScrollLayer 2, 0, -1
				VScrollCount = 0
			END IF
			Camera.y = Camera.y + 1
		END IF
		' Left
		IF (Player.x + 15) - Camera.x < 160 - 10 * Scroll AND Camera.x > 0 THEN
			IF ScrollMode = 1 OR ScrollMode = 3 THEN HScrollCount = HScrollCount + 1
			IF HScrollCount > ScrollRate THEN
				ScrollLayer 2, 1, 0
				HScrollCount = 0
			END IF
			Camera.x = Camera.x - 1
		END IF
		' Right
		IF (Player.x + 15) - Camera.x > 160 + 10 * Scroll AND Camera.x < (24 * MapXdim) - 320 THEN
			IF ScrollMode = 1 OR ScrollMode = 3 THEN HScrollCount = HScrollCount + 1
			IF HScrollCount > ScrollRate THEN
				ScrollLayer 2, -1, 0
				HScrollCount = 0
			END IF
			Camera.x = Camera.x + 1
		END IF
	NEXT Scroll
 
	IF Player.Death = FALSE THEN
		' Shoot routine
		Player.Shoot = 0
		IF DQBkey(56) AND sCount = 5 AND Player.Stat = 0 THEN
			IF Player.Look >= 0 THEN
				Bullet(NewBullet).yMove = 0
				IF Player.Dir = 0 THEN Bullet(NewBullet).x = Player.x + 30: Bullet(NewBullet).xMove = 4
				IF Player.Dir = 1 THEN Bullet(NewBullet).x = Player.x - 5: Bullet(NewBullet).xMove = -4
				SELECT CASE Player.Look
					CASE 0: Bullet(NewBullet).y = Player.y + 18
					CASE 1: Bullet(NewBullet).y = Player.y + 38
				END SELECT
			ELSE
				Bullet(NewBullet).xMove = 0
				Bullet(NewBullet).yMove = -4
				IF Player.Dir = 0 THEN Bullet(NewBullet).x = Player.x + 12
				IF Player.Dir = 1 THEN Bullet(NewBullet).x = Player.x + 11
				Bullet(NewBullet).y = Player.y - 5
			END IF
			Bullet(NewBullet).BulType = 1
			Bullet(NewBullet).Active = TRUE
			NewBullet = NewBullet + 1
			IF NewBullet > 100 THEN NewBullet = 0
			Player.Shoot = 1
			DS4QB.PlaySound 1
		END IF
	  
		' Handle user input
		Player.Look = 0
		Player.Walk = FALSE
		IF DQBkey(47) AND vHeld = FALSE THEN vSync = vSync + 1 AND 1: vHeld = TRUE
		IF DQBkey(47) = 0 AND vHeld THEN vHeld = FALSE
		IF DQBkey(KEYUP) THEN Player.Look = -1
		IF DQBkey(KEYDOWN) AND Player.Fall = FALSE THEN Player.Look = 1
		IF DQBkey(29) THEN Player.Stat = 0
		IF DQBkey(29) = 0 THEN jHeld = 0
		Player.xMove = 0
		IF Player.Look <= 0 THEN
			IF DQBkey(29) AND jHeld = FALSE THEN
				IF InCrap THEN Player.yMove = -6.2: Player.Jump = TRUE
				IF Player.yMove = 0 THEN Player.yMove = -6.2: Player.Jump = TRUE
				jHeld = TRUE
			END IF
			IF DQBkey(KEYLEFT) THEN Player.xMove = -2: Player.Dir = 1: Player.Walk = TRUE: Player.Stat = 0
			IF DQBkey(KEYRIGHT) THEN Player.xMove = 2: Player.Dir = 0: Player.Walk = TRUE: Player.Stat = 0
		END IF
		Player.Fall = TRUE: IF Player.yMove = 0 THEN Player.Fall = FALSE
	END IF
	IF Player.Stat = 0 THEN Player.yMove = Player.yMove + .2
	IF Player.Stat THEN Player.Look = 0
	IF InCrap THEN Player.xMove = Player.xMove / 1.8
	Player.y = Player.y + Player.yMove
	Player.x = Player.x + Player.xMove

	' Calculate start and end tile points
	StartX = Camera.x \ 24
	StartY = Camera.y \ 24
	IF StartX < 1 THEN StartX = 1
	IF StartY < 1 THEN StartY = 1
	EndX = StartX + 15
	EndY = StartY + 10
	IF EndX > MapXdim THEN EndX = MapXdim
	IF EndY > MapYdim THEN EndY = MapYdim
	' Draw background tiles
	DQBcopyLayer 2, 1
	FOR Layer = 1 TO 0 STEP -1
		DQBpeek VARSEG(LevelData(0)), VARPTR(LevelData(0)), 148480 + (15000 * Layer), 15000
		FOR Dy = StartY TO EndY
			FOR dx = StartX TO EndX
				' Calculate block position and tile offset
				BlockX = 24 * (dx - 1) - Camera.x
				BlockY = 24 * (Dy - 1) - Camera.y
				Offs = (dx - 1) + 150 * (Dy - 1)
				' Get tile data
				DEF SEG = VARSEG(LevelData(0))
				Tile = PEEK(VARPTR(LevelData(0)) + Offs)
				DEF SEG = VARSEG(TileFlag(Tile))
				DFlag = PEEK(VARPTR(TileFlag(Tile)) + 1)
				DEF SEG
				IF DFlag = 1 OR DFlag = 3 THEN DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), 580& * (Tile + TileFrame(Tile)), 580
				SELECT CASE DFlag
					CASE 1: RELsprite 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
					CASE 3: RELspriteTrans 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
				END SELECT
			NEXT dx
		NEXT Dy
	NEXT Layer

	' Process bullets
	FOR B = 0 TO 100
		IF Bullet(B).Active THEN
			' Move the bullet
			OldX = Bullet(B).x
			OldY = Bullet(B).y
			Bullet(B).x = Bullet(B).x + Bullet(B).xMove
			Bullet(B).y = Bullet(B).y + Bullet(B).yMove
			Angle = DQBangle(OldX, OldY, Bullet(B).x, Bullet(B).y)
			SELECT CASE Bullet(B).BulType
				CASE 1
					GETsprite 42, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
					DQBrPut 1, CINT(Bullet(B).x) - Camera.x, CINT(Bullet(B).y) - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), Angle, 100
				CASE 2
					GETsprite 43, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
					RELsprite 1, CINT(Bullet(B).x) - Camera.x, CINT(Bullet(B).y) - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			END SELECT
			' Collision detection
			PosX = Bullet(B).x \ 24
			PosY = Bullet(B).y \ 24
			HitFlag = GetHitFlag(GetTile(PosX, PosY, 0))
			SELECT CASE HitFlag
				CASE 1 TO 7
					Bullet(B).Active = FALSE
					SELECT CASE Bullet(B).BulType
						CASE 1
							' Put spark particles
							FOR P = 1 TO 10
								PxMove! = Bullet(B).xMove - Bullet(B).xMove * 2
								PyMove! = Bullet(B).yMove - Bullet(B).yMove * 2
								PutParticle Bullet(B).x + INT((5 - -5 + 1) * RND + -5), Bullet(B).y + INT((5 - -5 + 1) * RND + -5), PxMove!, PyMove!, CYELLOW
							NEXT P
							DS4QB.PlaySound INT((4 - 2 + 1) * RND + 2)
						CASE 2
							' Particlize the bullet
							Particlize CINT(Bullet(B).x), CINT(Bullet(B).y), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), -1
							DS4QB.PlaySound 13
					END SELECT
			END SELECT
			' Check if bullet hit sakura
			IF Player.Death = FALSE THEN
				Look = Player.Look
				IF Look < 0 THEN Look = 0
				IF Bullet(B).x > Player.x + 10 AND Bullet(B).x < Player.x + 20 THEN
					IF Bullet(B).y > Player.y + 20 * Look AND Bullet(B).y < Player.y + 62 THEN
						Bullet(B).Active = FALSE
						SELECT CASE Bullet(B).BulType
							CASE 1: Player.Health = Player.Health - 10
							CASE 2: Player.Health = Player.Health - 20
						END SELECT
						' Put BLOOD!
						FOR P = 1 TO 10
							PxMove! = Bullet(B).xMove - Bullet(B).xMove * 2
							PyMove! = Bullet(B).yMove - Bullet(B).yMove * 2
							PutParticle Bullet(B).x + INT((5 - -5 + 1) * RND + -5), Bullet(B).y + INT((5 - -5 + 1) * RND + -5), PxMove!, PyMove!, CHRED
						NEXT P
					END IF
				END IF
			END IF
			' Don't let the bullet escape the map!
			IF Bullet(B).x < -10 THEN Bullet(B).Active = FALSE
			IF Bullet(B).y < -10 THEN Bullet(B).Active = FALSE
			IF Bullet(B).x > 24 * MapXdim THEN Bullet(B).Active = FALSE
			IF Bullet(B).y > 24 * MapYdim THEN Bullet(B).Active = FALSE
		END IF
	NEXT B

	' Process enemies
	FOR E = 1 TO EnemyCount
		SELECT CASE Enemy(E).EnType
			CASE 1
				GETsprite 36 + 4, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				RELsprite 1, Enemy(E).x - Camera.x, Enemy(E).y - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				' Check if maid is hit
				FOR Check = 0 TO 100
					IF Bullet(Check).Active THEN
						IF Bullet(Check).x > Enemy(E).x - 11 AND Bullet(Check).x < Enemy(E).x + 21 THEN
							IF Bullet(Check).y > Enemy(E).y - 11 AND Bullet(Check).y < Enemy(E).y + 71 THEN
								Enemy(E).HitCount = Enemy(E).HitCount + 1
								IF Enemy(E).HitCount > 5 THEN
									Enemy(E).HitCount = 0
									Enemy(E).EnType = 2
									Particlize CINT(Enemy(E).x), CINT(Enemy(E).y), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), -1
									DS4QB.AddSound 6, CURRENT, CURRENT, CURRENT, CURRENT
									DS4QB.AddSound 7, CURRENT, CURRENT, CURRENT, CURRENT
									DS4QB.PlaySounds
								END IF
								Bullet(Check).Active = FALSE
								' Put BLOOD!
								FOR P = 1 TO 10
									PxMove! = Bullet(Check).xMove - Bullet(Check).xMove * 2
									PyMove! = Bullet(Check).yMove - Bullet(Check).yMove * 2
									PutParticle Bullet(Check).x + INT((5 - -5 + 1) * RND + -5), Bullet(Check).y + INT((5 - -5 + 1) * RND + -5), PxMove!, PyMove!, CHRED
								NEXT P
								EXIT FOR
							END IF
						END IF
					END IF
				NEXT Check
			CASE 2
				GETsprite 36 + 5, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				RELspriteTrans 1, Enemy(E).x - Camera.x, Enemy(E).y - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				' Follow sakura no matter what!
				MoveAngle = DQBangle(Enemy(E).x + 9, Enemy(E).y + 35, Player.x + 15, Player.y + 31) * 1.412
				Enemy(E).x = Enemy(E).x + 1 * ST(MoveAngle)
				Enemy(E).y = Enemy(E).y - 1 * CT(MoveAngle)
				' Shoot sakura at random intervals
				IF INT(RND * 10) + 1 = 5 THEN
					AimAngle = DQBangle(Enemy(E).x + 9, Enemy(E).y + 35, Player.x + 15, Player.y + 31) * 1.412
					Bullet(NewBullet).x = Enemy(E).x + 9
					Bullet(NewBullet).y = Enemy(E).y + 25
					Bullet(NewBullet).xMove = 4 * ST(AimAngle)
					Bullet(NewBullet).yMove = -4 * CT(AimAngle)
					Bullet(NewBullet).BulType = 2
					Bullet(NewBullet).Active = TRUE
					NewBullet = NewBullet + 1
					IF NewBullet > 100 THEN NewBullet = 0
					DS4QB.PlaySound 8
				END IF
				' Check if EVIL maid is hit
				FOR Check = 0 TO 100
					IF Bullet(Check).Active AND Bullet(Check).BulType = 1 THEN
						IF Bullet(Check).x > Enemy(E).x - 11 AND Bullet(Check).x < Enemy(E).x + 21 THEN
							IF Bullet(Check).y > Enemy(E).y - 11 AND Bullet(Check).y < Enemy(E).y + 71 THEN
								' Deflect sakura's shots!
								Bullet(Check).xMove = Bullet(Check).xMove - Bullet(Check).xMove * 2
								Bullet(Check).yMove = Bullet(Check).yMove - Bullet(Check).yMove * 2
								DS4QB.PlaySound 9
								EXIT FOR
							END IF
						END IF
					END IF
				NEXT Check
			CASE 3
				GETsprite 36 + (2 + Enemy(E).Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				RELsprite 1, Enemy(E).x - Camera.x, Enemy(E).y - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				' Check if worm found sakura
				IF Enemy(E).Found = FALSE THEN
					IF Player.Death = FALSE THEN
						IF Player.y > Enemy(E).y - 31 AND Player.y < Enemy(E).y + 31 THEN
							' Gotcha! Start shooting and chase her
							IF Enemy(E).Dir = 1 AND Player.x > Enemy(E).x + 43 THEN
								DS4QB.PlaySound 10
								Enemy(E).Found = TRUE
							ELSEIF Enemy(E).Dir = 0 AND Player.x < Enemy(E).x THEN
								DS4QB.PlaySound 10
								Enemy(E).Found = TRUE
							END IF
						END IF
					END IF
				ELSE
					' Shoot at sakura less often than the EVIL maid
					IF INT(RND * 40) + 1 = 20 THEN
						AimAngle = DQBangle(Enemy(E).x + 20, Enemy(E).y + 20, Player.x + 15, Player.y + 31) * 1.412
						Bullet(NewBullet).x = Enemy(E).x + 12
						Bullet(NewBullet).y = Enemy(E).y + 25
						Bullet(NewBullet).xMove = 4 * ST(AimAngle)
						Bullet(NewBullet).yMove = -4 * CT(AimAngle)
						Bullet(NewBullet).BulType = 2
						Bullet(NewBullet).Active = TRUE
						NewBullet = NewBullet + 1
						IF NewBullet > 100 THEN NewBullet = 0
						DS4QB.PlaySound 15
					END IF
					' Follow sakura
					IF Player.x > Enemy(E).x THEN Enemy(E).x = Enemy(E).x + 1: Enemy(E).Dir = 1
					IF Player.x < Enemy(E).x THEN Enemy(E).x = Enemy(E).x - 1: Enemy(E).Dir = 0
					' If sakura's dead, stop
					IF Player.Death THEN Enemy(E).Found = FALSE
				END IF
				' Check if worm is hit by bullet
				FOR Check = 0 TO 100
					IF Bullet(Check).Active THEN
						IF Bullet(Check).x > Enemy(E).x AND Bullet(Check).x < Enemy(E).x + 43 THEN
							IF Bullet(Check).y > Enemy(E).y AND Bullet(Check).y < Enemy(E).y + 48 THEN
								' Put BLOOD!
								IF Bullet(Check).BulType <> 2 THEN
									FOR P = 1 TO 10
										PxMove! = Bullet(Check).xMove - Bullet(Check).xMove * 2
										PyMove! = Bullet(Check).yMove - Bullet(Check).yMove * 2
										PutParticle Bullet(Check).x + INT((5 - -5 + 1) * RND + -5), Bullet(Check).y + INT((5 - -5 + 1) * RND + -5), PxMove!, PyMove!, CHRED
									NEXT P
									IF Enemy(E).HitCount < 10 THEN DS4QB.PlaySound 11
									Enemy(E).HitCount = Enemy(E).HitCount + 1
									Bullet(Check).Active = FALSE
								END IF
							END IF
						END IF
					END IF
				NEXT Check
				' Worm died, particlize its body the fun way!
				IF Enemy(E).HitCount > 10 THEN
					Particlize CINT(Enemy(E).x), CINT(Enemy(E).y), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), -1
					Enemy(E).EnType = 0
					DS4QB.PlaySound 12
				END IF
				' Worm's collision detection
				Enemy(E).yMove = Enemy(E).yMove + .2
				PosX = (Enemy(E).x + 21) \ 24
				PosY = (Enemy(E).y + 24) \ 24
				HitX = (Enemy(E).x + 21) - (24 * PosX)
				HitY = (Enemy(E).y + 24) - (24 * PosY)
				' Floor
				HitFlag = GetHitFlag(GetTile(PosX, PosY + 1, 0))
				SELECT CASE HitFlag
					CASE 1 TO 7
						Enemy(E).yMove = 0
						Enemy(E).y = (24 * PosY) - 24
				END SELECT
				IF HitFlag = 6 THEN Enemy(E).x = Enemy(E).x - .5
				IF HitFlag = 7 THEN Enemy(E).x = Enemy(E).x + .5
				' Sides
				HitFlag = GetHitFlag(GetTile(PosX - 1, PosY, 0))
				SELECT CASE HitFlag
					CASE 1 TO 7: Enemy(E).x = Enemy(E).x + 1
				END SELECT
				HitFlag = GetHitFlag(GetTile(PosX + 1, PosY, 0))
				SELECT CASE HitFlag
					CASE 1 TO 7: Enemy(E).x = Enemy(E).x - 1
				END SELECT
				Enemy(E).y = Enemy(E).y + Enemy(E).yMove
		END SELECT
	NEXT E

	' Sakura's collision detection code
	PosX = (Player.x + 16) \ 24
	PosY = (Player.y + 31) \ 24
	HitX = (Player.x + 16) - (24 * PosX)
	HitY = (Player.y + 31) - (24 * PosY)
	IF PosY < 1 THEN PosY = 1
	' Floor detection
	HitFlag = GetHitFlag(GetTile(PosX, PosY + 2, 0))
	SELECT CASE HitFlag
		CASE 1, 6, 7
			IF HitY > 16 THEN
				Player.y = (24 * PosY) - 14
				'IF Player.Fall THEN SOUND 200, 1
				Player.Fall = FALSE
				Player.Jump = FALSE
				Player.yMove = 0
			END IF
	END SELECT
	IF HitFlag = 6 AND HitY > 16 THEN Player.x = Player.x - .5
	IF HitFlag = 7 AND HitY > 16 THEN Player.x = Player.x + .5
	' Emergency floor detection (to prevent player from falling off floors)
	HitFlag = GetHitFlag(GetTile(PosX, PosY + 1, 0))
	SELECT CASE HitFlag
		CASE 1, 6, 7
			Player.y = (24 * (PosY - 1)) - 14
			'IF Player.Fall THEN SOUND 200, 1
			Player.yMove = 0
			Player.Jump = FALSE
			Player.Fall = FALSE
	END SELECT
	' Slope detection
	sPosX = (Player.x + 16) \ 24
	sPosY = (Player.y + 31) \ 24
	sHitX! = (Player.x + 16) - (24 * sPosX)
	sHitY! = (Player.y + 31) - (24 * sPosY)
	HitFlag = GetHitFlag(GetTile(sPosX, sPosY + 1, 0))
	SELECT CASE HitFlag
		CASE 2
			IF sHitY! > (25 - sHitX!) - 10 THEN
				sHitY! = (25 - sHitX!) - 10
				Player.yMove = 0
				'IF Player.Fall THEN SOUND 200, 1
				Player.Fall = FALSE
				Player.Jump = FALSE
			END IF
		CASE 3
			IF sHitY! > sHitX! - 10 THEN
				sHitY! = sHitX! - 10
				Player.yMove = 0
				'IF Player.Fall THEN SOUND 200, 1
				Player.Fall = FALSE
				Player.Jump = FALSE
			END IF
		CASE 13
			IF sHitY! > (25 - sHitX!) + 14 THEN
				sHitY! = (25 - sHitX!) + 14
				Player.yMove = 0
				'IF Player.Fall THEN SOUND 200, 1
				Player.Fall = FALSE
				Player.Jump = FALSE
			END IF
		CASE 14
			IF sHitY! > sHitX! + 14 THEN
				sHitY! = sHitX! + 14
				Player.yMove = 0
				'IF Player.Fall THEN SOUND 200, 1
				Player.Fall = FALSE
				Player.Jump = FALSE
			END IF
	END SELECT
	Player.x = ((24 * sPosX) + sHitX!) - 16
	Player.y = ((24 * sPosY) + sHitY!) - 31
	' Ceiling detection
	HitFlag = GetHitFlag(GetTile(PosX, PosY - 2, 0))
	SELECT CASE HitFlag
		CASE 1: IF HitY < 7 THEN Player.y = (24 * PosY) - 23: Player.yMove = 1
	END SELECT
	' Side detection
	Hit = FALSE
	FOR L = -1 TO 1
		IF Hit = FALSE THEN
			' Left
			HitFlag = GetHitFlag(GetTile(PosX - 1, PosY + L, 0))
			SELECT CASE HitFlag
				CASE 1: IF HitX < 5 THEN Player.x = (24 * PosX) - 10: Player.Walk = FALSE: Hit = TRUE
			END SELECT
			' Right
			HitFlag = GetHitFlag(GetTile(PosX + 1, PosY + L, 0))
			SELECT CASE HitFlag
				CASE 1: IF HitX > 16 THEN Player.x = (24 * PosX): Player.Walk = FALSE: Hit = TRUE
			END SELECT
			' Middle (for water and slime)
			HitFlag = GetHitFlag(GetTile(PosX, PosY + L, 0))
			SELECT CASE HitFlag
				CASE 8: Player.x = Player.x - .5
				CASE 9: Player.x = Player.x + .5
			END SELECT
		END IF
	NEXT L
	' Crap sand detection
	InCrap = FALSE
	CrapLevel = 0
	FOR L = -1 TO 1
		HitFlag = GetHitFlag(GetTile(PosX, PosY + L, 0))
		IF HitFlag = 12 THEN InCrap = TRUE: CrapLevel = CrapLevel + 1: Player.Jump = FALSE
	NEXT L
	IF InCrap = TRUE AND Player.Jump = FALSE THEN Player.yMove = .1: Player.Fall = FALSE
	IF CrapLevel = 3 AND Player.Death = FALSE THEN Player.Death = TRUE
	IF Player.Death = FALSE THEN
		' Ladder detection
		OnLadder = FALSE
		FOR L = -1 TO 1
			HitFlag = GetHitFlag(GetTile(PosX, PosY + L, 0))
			IF HitFlag = 10 THEN OnLadder = TRUE
		NEXT L
		IF OnLadder THEN
			IF DQBkey(KEYUP) THEN Player.y = Player.y - 2: Player.Stat = 1: Player.yMove = 0: Player.Jump = FALSE: Player.Walk = TRUE
			IF DQBkey(KEYDOWN) THEN Player.y = Player.y + 2: Player.Stat = 1: Player.yMove = 0: Player.Jump = FALSE: Player.Walk = TRUE
		ELSE
			Player.Stat = 0
		END IF
		' Bars detection
		HitFlag = GetHitFlag(GetTile(PosX, PosY - 1, 0))
		IF HitFlag = 11 AND HitY > 16 AND HitY < 24 AND DQBkey(KEYDOWN) = 0 THEN
			Player.y = (24 * (PosY)) - 12
			Player.yMove = 0
			Player.Stat = 2
		END IF
	END IF
  
	' Process tags
	IF Player.Death = FALSE THEN
		sHeld = FALSE
		FOR D = 1 TO TagCount
			GetTagData D, x, y, HTag, LTag
			IF PosX = x AND PosY = y THEN
				SELECT CASE LTag
					CASE 1
						IF DQBkey(KEYSPACE) THEN
							Tile = GetTile(x, y, 0)
							SetTile x, y, 0, Tile + 1
							SetTagData D, x, y, 0, 0
							ActivateTags HTag
							DS4QB.PlaySound 5
						END IF
					CASE 2
						ActivateTags HTag
						SetTagData D, x, y, 0, 0
				END SELECT
			END IF
		NEXT D
	END IF

	' Put sakura sprites
	IF Player.Death = FALSE THEN
		IF Player.Stat = 0 THEN
			IF Player.Look <= 0 THEN
				Px = CINT(Player.x): Py = CINT(Player.y)
			ELSE
				Px = CINT(Player.x): Py = CINT(Player.y) + 20
			END IF
			' Put body section
			SELECT CASE Player.Look
				CASE -1: GETsprite 3 + (17 * Player.Dir) + Player.Shoot, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				CASE ELSE: GETsprite 1 + (17 * Player.Dir) + Player.Shoot, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			END SELECT
			RELsprite 1, Px - Camera.x, Py - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			' Put leg section
			IF Player.Look <= 0 THEN
				IF Player.Walk AND Player.Fall = FALSE THEN
					GETsprite (10 + WalkFrame + (17 * Player.Dir)), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				ELSEIF Player.Jump = FALSE THEN
					GETsprite 9 + (17 * Player.Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				ELSE
					GETsprite 14 + (17 * Player.Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				END IF
			ELSE
				GETsprite 16 + (17 * Player.Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			END IF
			RELsprite 1, Px - Camera.x, (Py + 31) - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
		ELSEIF Player.Stat = 1 THEN
			GETsprite 35 + (WalkFrame MOD 2), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			RELsprite 1, (Player.x + 6) - Camera.x, Player.y - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
		ELSEIF Player.Stat = 2 THEN
			Px = CINT(Player.x): Py = CINT(Player.y)
			GETsprite 5 + (WalkFrame + (17 * Player.Dir)), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			RELsprite 1, Px - Camera.x, Py - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			GETsprite 15 + (17 * Player.Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			RELsprite 1, Px - Camera.x, (Py + 31) - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
		END IF
	ELSE
		GETsprite 17 + (17 * Player.Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
		RELsprite 1, CINT(Player.x) - Camera.x, CINT(Player.y + 49) - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
	END IF

	' Process items
	FOR P = 1 TO ItemCount
		IF Item(P).iType THEN
			SELECT CASE Item(P).iType
				CASE 1: GETsprite 36 + 8, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
				CASE 2: GETsprite 36 + 9, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			END SELECT
			IF Item(P).x > Player.x AND Item(P).x < Player.x + 31 THEN
				IF Item(P).y > Player.y AND Item(P).y < Player.y + 62 THEN
					SELECT CASE Item(P).iType
						CASE 1: Player.Health = Player.Health + 20
						CASE 2: Player.Health = Player.Health + 40
					END SELECT
					DS4QB.PlaySound 17
					Item(P).iType = 0
				END IF
			END IF
			RELsprite 1, Item(P).x - Camera.x, Item(P).y - Camera.y, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
		END IF
	NEXT P

	' Process particles
	FOR P = 0 TO 100
		IF Particle(P).Active THEN
			SELECT CASE Particle(P).pType
				CASE 0 TO 255: DQBpset 1, Particle(P).x - Camera.x, Particle(P).y - Camera.y, Particle(P).pType
			END SELECT
			IF Particle(P).y - Camera.y > 200 THEN Particle(P).Active = FALSE
			Particle(P).x = Particle(P).x + Particle(P).xMove
			Particle(P).y = Particle(P).y + Particle(P).yMove
			Particle(P).yMove = Particle(P).yMove + .2
		END IF
	NEXT P

	' Draw foreground tiles
	FOR Layer = 1 TO 0 STEP -1
		DQBpeek VARSEG(LevelData(0)), VARPTR(LevelData(0)), 148480 + (15000 * Layer), 15000
		FOR Dy = StartY TO EndY
			FOR dx = StartX TO EndX
				' Calculate block position and tile offset
				BlockX = 24 * (dx - 1) - Camera.x
				BlockY = 24 * (Dy - 1) - Camera.y
				Offs = (dx - 1) + 150 * (Dy - 1)
				' Get tile data
				DEF SEG = VARSEG(LevelData(0))
				Tile = PEEK(VARPTR(LevelData(0)) + Offs)
				DEF SEG = VARSEG(TileFlag(Tile))
				DFlag = PEEK(VARPTR(TileFlag(Tile)) + 1)
				DEF SEG
				IF DFlag = 0 OR DFlag = 2 THEN DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), 580& * (Tile + TileFrame(Tile)), 580
				SELECT CASE DFlag
					CASE 0: RELsprite 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
					CASE 2: RELspriteTrans 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
				END SELECT
			NEXT dx
		NEXT Dy
	NEXT Layer

	' Put engine logo
	IF LogoZoom < 100 AND LogoTime = 0 THEN
		LogoZoom = LogoZoom + 1
		LogoAngle = 2.55 * LogoZoom
	ELSEIF LogoZoom = 100 AND LogoTime < 120 THEN
		LogoTime = LogoTime + 1
	ELSEIF LogoTime = 120 AND LogoZoom > 0 THEN
		LogoZoom = LogoZoom - 1
		LogoAngle = 2.55 * LogoZoom
	END IF
	IF LogoZoom > 0 THEN
		GETsprite 37, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
		DQBrPut 1, 112, 52, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), LogoAngle + 1, LogoZoom
	END IF

	DQBprint 1, "FPS:" + LTRIM$(STR$(FPS)), 1, 1, CHWHITE
	DQBprint 1, "HEALTH:" + LTRIM$(STR$(Player.Health)), CENTERED, 180, CHWHITE
	IF Player.Health <= 0 AND Player.Death = FALSE THEN Player.Death = TRUE: Player.xMove = 0

	' Do death animation
	IF Player.Death THEN
		Player.Stat = 0
		' Clear EVIL maids when sakura dies
		IF dCount = 0 THEN
			DS4QB.PlaySound 14
			GETsprite 36 + 5, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			FOR C = 1 TO EnemyCount
				IF Enemy(C).EnType = 2 THEN
					Enemy(C).EnType = 0
					Particlize CINT(Enemy(C).x), CINT(Enemy(C).y), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), -1
				END IF
			NEXT C
		END IF
		DQBprint 1, "--DEATH!--", CENTERED, 99, CRED
		DQBprint 1, "--DEATH!--", CENTERED, 98, CHRED
		DQBget 1, 0, 0, 319, 199, VARSEG(SCRN(0)), VARPTR(SCRN(0))
		DQBclearLayer 1
		DQBrPut 1, 0, 0, VARSEG(SCRN(0)), VARPTR(SCRN(0)), dCount MOD 256, 100 + dCount
		dCount = dCount + 6
		IF dCount > 510 THEN
			dCount = 0
			Player.Death = FALSE
			Player.Health = 100
			Player.x = pStartX
			Player.y = pStartY
			Player.xMove = 0
			Player.yMove = 0
			Player.Dir = 0
			GOSUB FixCamera
			GETsprite 17 + (17 * Player.Dir), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			Particlize CINT(Player.x), CINT(Player.y), VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), CHWHITE
			DS4QB.PlaySound 8
		END IF
	END IF

	IF vSync THEN DQBwait 1
	DQBcopyLayer 1, VIDEO

	' Do animations
	FOR Animate = 0 TO 255
		' Get animation flags
		DEF SEG = VARSEG(TileAnim(Animate))
		Anim = PEEK(VARPTR(TileAnim(Animate)))
		AnmF = PEEK(VARPTR(TileAnim(Animate)) + 1)
		' Get animation stats
		DEF SEG = VARSEG(AnimData(Animate))
		AnimDir = PEEK(VARPTR(AnimData(Animate))) - 1
		AnimCnt = PEEK(VARPTR(AnimData(Animate)) + 1)
		AnimCnt = AnimCnt + 1
		IF AnimCnt > AnimRate(Animate) THEN
			SELECT CASE Anim
				CASE 1
					IF TileFrame(Animate) = 0 THEN AnimDir = 1
					IF TileFrame(Animate) = AnmF THEN AnimDir = -1
					TileFrame(Animate) = TileFrame(Animate) + AnimDir
				CASE 2
					TileFrame(Animate) = TileFrame(Animate) + 1
					IF TileFrame(Animate) > AnmF THEN TileFrame(Animate) = 0
				CASE 3
					TileFrame(Animate) = TileFrame(Animate) - 1
					IF TileFrame(Animate) < (AnmF - AnmF * 2) THEN TileFrame(Animate) = 0
			END SELECT
			AnimCnt = 1
		END IF
		' Set animation stats
		POKE VARPTR(AnimData(Animate)), AnimDir + 1
		POKE VARPTR(AnimData(Animate)) + 1, AnimCnt
		DEF SEG
	NEXT Animate

	' Handle walking animation frames
	IF wCount = 10 THEN
		IF Player.Walk THEN
			WalkFrame = WalkFrame + 1
			IF WalkFrame > 3 THEN WalkFrame = 0
		END IF
	END IF

	' Calulate FPS
	IF CLNG(TIMER) > OldTime& THEN
		IF InCrap THEN DS4QB.PlaySound 16
		OldTime& = CLNG(TIMER)
		FPS = Loops
		Loops = 0
	END IF
	Loops = Loops + 1

	sCount = sCount + 1
	IF sCount > 5 THEN sCount = 0
	wCount = wCount + 1
	IF wCount > 10 THEN wCount = 0

LOOP UNTIL DQBkey(KEYESC)
EndIt:
DS4QB.Close
DQBclose
END

FixCamera:
	DO
		Scrolling = FALSE
		FOR Scroll = 1 TO 50
			IF (Player.y + 31) - Camera.y < 100 - (10 * Scroll) AND Camera.y > 0 THEN Camera.y = Camera.y - 1: Scrolling = TRUE
			IF (Player.y + 31) - Camera.y > 100 + (10 * Scroll) AND Camera.y < (24 * MapYdim) - 200 THEN Camera.y = Camera.y + 1: Scrolling = TRUE
			IF (Player.x + 15) - Camera.x < 160 - (10 * Scroll) AND Camera.x > 0 THEN Camera.x = Camera.x - 1: Scrolling = TRUE
			IF (Player.x + 15) - Camera.x > 160 + (10 * Scroll) AND Camera.x < (24 * MapXdim) - 320 THEN Camera.x = Camera.x + 1: Scrolling = TRUE
		NEXT Scroll
	LOOP UNTIL Scrolling = FALSE
RETURN

ErrorHandler:
	IF ERR = 70 THEN RESUME
	DQBinitText
	CLS : PRINT "Some stupid error #"; LTRIM$(STR$(ERR))
	DS4QB.Close
	DQBclose
	END
RESUME

REM $STATIC
SUB ActivateTags (HTag)

FOR A = 1 TO TagCount
	GetTagData A, xPos, yPos, cHTag, cLTag
	IF HTag = cHTag THEN
		SELECT CASE cLTag
			CASE 1
				Tile = GetTile(xPos, yPos, 0)
				SetTile xPos, yPos, 0, Tile + 1
				SetTagData A, xPos, yPos, 0, 0
			CASE 3
				SetTile xPos, yPos, 0, 0
			CASE 4
				Tile = GetTile(xPos, yPos, 0)
				SetTile xPos, yPos, 0, Tile - 1
		END SELECT
	END IF
NEXT A

END SUB

FUNCTION GetHitFlag (Tile)
 
SHARED TileFlag()

DEF SEG = VARSEG(TileFlag(Tile))
GetHitFlag = PEEK(VARPTR(TileFlag(Tile)))
DEF SEG

END FUNCTION

FUNCTION GETmemRequired& (GETfile$)

OPEN GETfile$ FOR BINARY AS #1
IF LOF(1) = 0 THEN
	CLOSE #1: KILL GETfile$
	GETmemRequired& = -1
	EXIT FUNCTION
END IF
IF INPUT$(4, 1) <> "GET1" THEN
	CLOSE #1: GETmemRequired& = -2
	EXIT FUNCTION
END IF

BoxCount = ASC(INPUT$(1, 1))
NameLength = ASC(INPUT$(1, 1))
SEEK #1, NameLength + (LOC(1) + 1)

FOR Calc = 1 TO BoxCount
	x1 = CVI(INPUT$(2, 1))
	y1 = CVI(INPUT$(2, 1))
	x2 = CVI(INPUT$(2, 1))
	y2 = CVI(INPUT$(2, 1))
	Xsize = x2 - x1
	Ysize = y2 - y1
	MemRequired& = MemRequired& + (Xsize * Ysize) + 4
NEXT Calc
CLOSE #1

GETmemRequired& = MemRequired&

END FUNCTION

FUNCTION GETprocess (GETfile$, TempLayer, TempSeg, TempOff, TempPal$)

OPEN GETfile$ FOR BINARY AS #1
IF LOF(1) = 0 THEN
	CLOSE #1: KILL GETfile$
	GETprocess = -1: EXIT FUNCTION
END IF
IF INPUT$(4, 1) <> "GET1" THEN
	CLOSE #1: GETprocess = -2
	EXIT FUNCTION
END IF

SpriteCount = ASC(INPUT$(1, 1))
IMGfile$ = INPUT$(ASC(INPUT$(1, 1)), 1)
IF DQBloadImage(TempLayer, 0, 0, IMGfile$, TempPal$, 0, 0) THEN
	CLOSE #1: GETprocess = -3
	EXIT FUNCTION
END IF

Offs& = SpriteDataOffs
FOR Calc = 1 TO NewSpriteIndex: Offs& = Offs& + SpriteSize(Calc): NEXT Calc
FOR G = 1 TO SpriteCount
	NewSpriteIndex = NewSpriteIndex + 1
	x1 = CVI(INPUT$(2, 1))
	y1 = CVI(INPUT$(2, 1))
	x2 = CVI(INPUT$(2, 1))
	y2 = CVI(INPUT$(2, 1))
	DQBget TempLayer, x1, y1, x2, y2, TempSeg, TempOff
	SpriteSize(NewSpriteIndex) = DQBsize(x1, y1, x2, y2)
	SpriteOffs(NewSpriteIndex) = Offs&
	DQBpoke TempSeg, TempOff, Offs&, SpriteSize(NewSpriteIndex)
	Offs& = Offs& + SpriteSize(NewSpriteIndex)
NEXT G

CLOSE #1

END FUNCTION

SUB GETsprite (Index, SpriteSeg, SpriteOff)
DQBpeek SpriteSeg, SpriteOff, SpriteOffs(Index), SpriteSize(Index)
END SUB

SUB GetTagData (Tag, xPos, yPos, HTag, LTag)

SHARED Tags() AS LONG

DEF SEG = VARSEG(Tags(Tag))
xPos = PEEK(VARPTR(Tags(Tag)) + 0)
yPos = PEEK(VARPTR(Tags(Tag)) + 1)
HTag = PEEK(VARPTR(Tags(Tag)) + 2)
LTag = PEEK(VARPTR(Tags(Tag)) + 3)
DEF SEG

END SUB

FUNCTION GetTile (x, y, Layer)

SHARED LevelData()

Offs = x + 150 * y
DQBpeek VARSEG(LevelData(0)), VARPTR(LevelData(0)), 148480 + (15000 * Layer), 15000
DEF SEG = VARSEG(LevelData(0))
GetTile = PEEK(VARPTR(LevelData(0)) + Offs)
DEF SEG

END FUNCTION

FUNCTION GetWinMode

DIM inreg AS RegTypeX, outreg AS RegTypeX
inreg.ax = &H160A
CALL INTERRUPT(&H2F, inreg, outreg)
IF outreg.ax <> 0 THEN WinMode = 0 ELSE WinMode = outreg.cx
IF WinMode = 0 AND ENVIRON$("OS") = "Windows_NT" THEN WinMode = 4
GetWinMode = WinMode

END FUNCTION

SUB Particlize (x, y, SpriteSeg, SpriteOff, Col)

DEF SEG = SpriteSeg
xDim$ = CHR$(PEEK(SpriteOff + 0)) + CHR$(PEEK(SpriteOff + 1))
yDim$ = CHR$(PEEK(SpriteOff + 2)) + CHR$(PEEK(SpriteOff + 3))

xDim = CVI(xDim$) \ 8
yDim = CVI(yDim$)

FOR Py = 1 TO yDim
	FOR Px = 1 TO xDim
		IF INT((15 - 1 + 1) * RND + 1) = 7 THEN
			Offs& = (Px - 1) + CLNG(xDim) * (Py - 1)
			Pix = PEEK((SpriteOff + 4) + Offs&)
			IF Pix THEN
				IF Col = -1 THEN
					PutParticle CSNG(x + (Px - 1)), CSNG(y + (Py - 1)), 0, 0, Pix
				ELSE
					PutParticle CSNG(x + (Px - 1)), CSNG(y + (Py - 1)), 0, 0, Col
				END IF
			END IF
		END IF
	NEXT Px
NEXT Py

DEF SEG

END SUB

SUB PutParticle (x!, y!, PxMove!, PyMove!, pType)

SHARED NewParticle, Particle() AS ParticleType

IF PxMove! = 0 THEN PxMove! = INT((3.5 - -3.5 + 1) * RND + -3.5)
IF PyMove! = 0 THEN PyMove! = INT((3.5 - -3.5 + 1) * RND + -3.5)
					 
Particle(NewParticle).x = x!
Particle(NewParticle).y = y!
Particle(NewParticle).xMove = PxMove!
Particle(NewParticle).yMove = PyMove!
Particle(NewParticle).Active = TRUE
Particle(NewParticle).pType = pType
NewParticle = NewParticle + 1
IF NewParticle > 100 THEN NewParticle = 0

END SUB

SUB ScrollLayer (Layer, xMove, yMove)

SHARED LineBuffer()

DEF SEG = VARSEG(LineBuffer(0))
SELECT CASE xMove
	CASE IS < 0
		FOR Copy = 0 TO 199 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, 0, Copy)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, 0, Copy + 1)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, 0, Copy + 2)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, 0, Copy + 3)
		NEXT
		DQBscroll Layer, -1, 0
		FOR Paste = 0 TO 199 STEP 4
			DQBpset Layer, 319, Paste, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, 319, Paste + 1, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, 319, Paste + 2, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, 319, Paste + 3, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
	CASE IS > 0
		FOR Copy = 0 TO 199 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, 319, Copy)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, 319, Copy + 1)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, 319, Copy + 2)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, 319, Copy + 3)
		NEXT
		DQBscroll Layer, 1, 0
		FOR Paste = 0 TO 199 STEP 4
			DQBpset Layer, 0, Paste, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, 0, Paste + 1, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, 0, Paste + 2, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, 0, Paste + 3, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
END SELECT
SELECT CASE yMove
	CASE IS < 0
		FOR Copy = 0 TO 319 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, Copy, 0)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, Copy + 1, 0)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, Copy + 2, 0)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, Copy + 3, 0)
		NEXT
		DQBscroll Layer, 0, -1
		FOR Paste = 0 TO 319 STEP 4
			DQBpset Layer, Paste, 199, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, Paste + 1, 199, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, Paste + 2, 199, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, Paste + 3, 199, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
	CASE IS > 0
		FOR Copy = 0 TO 319 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, Copy, 199)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, Copy + 1, 199)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, Copy + 2, 199)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, Copy + 3, 199)
		NEXT
		DQBscroll Layer, 0, 1
		FOR Paste = 0 TO 319 STEP 4
			DQBpset Layer, Paste, 0, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, Paste + 1, 0, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, Paste + 2, 0, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, Paste + 3, 0, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
END SELECT
DEF SEG

END SUB

SUB SetTagData (Tag, xPos, yPos, HTag, LTag)

SHARED Tags() AS LONG

DEF SEG = VARSEG(Tags(Tag))
POKE VARPTR(Tags(Tag)) + 0, xPos
POKE VARPTR(Tags(Tag)) + 1, yPos
POKE VARPTR(Tags(Tag)) + 2, HTag
POKE VARPTR(Tags(Tag)) + 3, LTag
DEF SEG

END SUB

SUB SetTile (x, y, Layer, Tile)

SHARED LevelData()

Offs = x + 150 * y
DQBpeek VARSEG(LevelData(0)), VARPTR(LevelData(0)), 148480 + (15000 * Layer), 15000
DEF SEG = VARSEG(LevelData(0))
POKE VARPTR(LevelData(0)) + Offs, Tile
DQBpoke VARSEG(LevelData(0)), VARPTR(LevelData(0)), 148480 + (15000 * Layer), 15000
DEF SEG

END SUB

