defint a-z
option explicit
option byval

'$include: 'fbgfx.bi'

const xres = 320
const yres = 240
const bpp = 32
const tilesize = 32
const maxx% = (xres/tilesize)*32
const maxy% = (yres/tilesize)*32
const fps = 60
const maxmines = 256
const deminermaxv = 4/fps
const dtilehitmaxv = 32/fps
const demineraccel = (2.75/fps)/(fps/3)
const gravity = (1.1/fps)/(fps/3)
const maxstars = 320
const maxlargestars = 3
const starlevels = 3
const pi = 3.141593
const boosttime = 0.5
const shieldinc = 100/(fps*60)
const shielddamage = 8.5 / deminermaxv
const minemovetime = 1.0
const mineshieldinc = 33
const warningTime = 2.0
const maxdtiles = 256
const maxframes = 256
const dtileforcefactor = 0.75
const levels = 11
const WARNING = 0
const COUNTING = 1

#define CENOF(txt) fix(160 - len(ltrim$(rtrim$(txt)))*7/2)

type levelT
	lname as string * 20
	ltime as integer
	ldtInterval as double
	ldtforce as double
	lgravfactor as double
end type

type deminerT
	x as double
	y as double
	bx as double
	by as double
	xv as double
	yv as double
	xa as double
	ya as double
	maxv as double
	lastboost as double
	lastslow as double
	lastabsorb as double
	bt as double
	px(0 to 9) as double
	py(0 to 9) as double
	shield as double
end type

type starT
	x as integer
	y as integer
	img as integer
	clr as uinteger
end type

type mineT
	inuse as integer
	x as double
	y as double
end type

type moveMineT
	inuse as integer
	x as double
	y as double
	st as double
end type

type dtileT
	inuse as integer
	frames as integer
	tile as integer
	frmx(0 to maxframes) as double
	frmy(0 to maxframes) as double
end type

type dtileXT
	xv as double
	yv as double
	x as double
	y as double
	onscreen as integer
	lt as double
end type

type optionsT
	musicVol as integer
	soundVol as integer
	graphicsDetail as integer
	firstRun as integer
	lastMusic as string * 128
	beatGame as integer
	compMode as integer
	'
	' Control settings
	'
end type

declare sub renderMap ( camx as double, camy as double, aa as integer )
declare sub renderMines ( camx as double, camy as double )
declare sub renderDtiles ( camx as double, camy as double, aa as integer )
declare sub renderDtilesStep ( camx as double, camy as double, j as integer )
declare sub cameraFocus ( byref camx as double, byref camy as double, fx as double, fy as double )
declare sub toScreenPos ( camx as double, camy as double, obx as double, oby as double, _
 						  byref sx as integer, byref sy as integer )
declare sub capVel ( byref xv as double, byref yv as double, maxv as double )
declare function collision ( x as integer, y as integer, image as integer ptr, byref cx as double, byref cy as double )
declare function mineCollision ( px as double, py as double, rad as double ) as integer
declare sub lightningRay ( camx as double, camy as double, ox as double, oy as double, _
					       dx as double, dy as double, t as double, clr as integer, disipate as double )
declare sub drawlazer ( byval x1 as integer, byval y1 as integer, byval x2 as integer, byval y2 as integer, _
        	            byval inr as integer, byval ing as integer, byval inb as integer, _
               		    byval outr as integer, byval outg as integer, byval outb as integer, _
		                byval opacity as double, byval oopacity as double )
declare	sub printn ( digits as integer, n as integer, x as integer, y as integer, colour as uinteger )
declare sub printt (ns as string, x as integer, y as integer, colour as uinteger )
declare sub printtb ( buffer as integer ptr, ns as string, x as integer, y as integer, colour as uinteger )
declare sub printt2 (ns as string, x as integer, y as integer, colour as uinteger )
declare sub printtb2 ( buffer as integer ptr, ns as string, x as integer, y as integer, colour as uinteger )
declare sub initStars ( )
declare sub renderStars ( camx as double, camy as double )
declare sub clsBb ( )
declare sub flipBb ( )
declare function cancelKey ( ) as integer
declare function myblit ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger
declare function myblit2 ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger
declare function brightBlend ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger
declare function brightBlend2 ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger
declare function makeTime ( t as double ) as string

common shared backBuffer as integer ptr

'$include: 'noise.bas'
'$include: 'fire.bas'
'$include: 'sound.bas'

dim shared as integer tiles
redim shared map(0 to maxx-1, 0 to maxy-1) as integer
redim shared minemap(0 to maxx-1, 0 to maxy-1, 0 to 9) as unsigned byte
redim shared tile(1, 0 to 32*32+4) as integer
redim shared mines(0 to maxmines-1) as mineT
redim shared movemines(0 to maxmines-1) as moveMineT
redim shared deminer(0 to 3, 0 to 16*16+4) as integer
redim shared antimine(0 to 0, 0 to 12*12+4) as integer
redim shared screens(0 to 1, 0 to 320*240*4) as integer
redim shared dtile(1 to 1) as dtileT, dtileX(1 to 1) as dtileXT
redim shared fontA(0 to 658*9+4) as integer, font(0 to 255, 0 to 7*9+4) as integer
redim shared font2(0 to 255, 0 to 7*9+4) as integer
redim shared star(0 to starlevels-1, 0 to maxstars-1) as starT
dim shared as integer ptr starImg(0 to 4)
dim shared as double cofx, cofy, copx, copy
dim shared as integer collCount
dim shared cBuffer as integer ptr
'dim shared backBuffer as integer ptr
dim newCompMode as integer
dim as string filename
dim as integer i, j, k, f
dim as integer ix, iy, ix2, iy2
dim player as deminerT, bplayer as deminerT
dim as double camx, camy
dim as integer fadeout=0, fader, fadeg, fadeb, red, blue, green, clr
dim as double fadetime
dim as integer ptr scrptr
dim as double ang, dist
dim as double t, t1, t2, t3, t4
dim as double xd, yd, bx, by
dim as integer minecount, dtilecount
dim as double warningStart, completeStart
dim shared as double timeStart, framestart
dim shared as integer levelTime = 5*60
dim shared as double dtileinterval = 1.0, dtileforce = 1.5
dim shared as double gravityFactor = 1.0
dim shared as levelT level
dim shared as levelT levelInfo(1 to levels)
dim shared black as integer ptr
dim shared as double storyFade, menuFade
dim cursor(16*16+4) as integer
dim as integer mousex, mousey, buttons, lmousex, lmousey
dim as integer levelNo, newHighScore
dim as integer frameCount
dim as double bestTime(1 to levels)
dim as integer help, lbuttonhold, optMenu
dim shared as optionsT options
dim as integer lastSelected
dim as double lastCollision
dim as integer warningSound

dim lastLargeNumber as integer = -1

dim as integer completeFlag = 0
dim as integer justStart = 1
dim as string theDir

dim shared story = 1

randomize timer

f = freefile
open "options.dat" for binary as #f
	if lof(f) = 0 then
		options.soundVol = 80
		options.musicVol = 80
		options.graphicsDetail = 2
		options.firstRun = 1
		options.compMode = 1
		put #f, , options
	else
		get #f, , options
	end if
close #f

if options.compMode then
	screenres xres*2, yres*2, bpp, 1, &h1 ' 640x480
else
	screenres xres, yres, bpp, 1, &h1 '320x240
end if
screenset 0, 0
setmouse , , 0

newCompMode = options.compMode

soundInit ( options.soundVol, options.musicVol )

cBuffer = imagecreate(xres, yres)
backBuffer = imagecreate(xres, yres)
black = imagecreate(xres, yres, 0)
dim as integer ptr fimg
fimg = imagecreate(xres, yres, 0)

bload "graphics/" + "deminer1.bmp", varptr(deminer(0, 0))
bload "graphics/" + "deminer2.bmp", varptr(deminer(1, 0))
bload "graphics/" + "deminer3.bmp", varptr(deminer(2, 0))
bload "graphics/" + "deminer4.bmp", varptr(deminer(3, 0))
bload "graphics/" + "antimine1.bmp", varptr(antimine(0, 0))
bload "graphics/" + "warning.bmp", varptr(screens(WARNING, 0))

bload "graphics/" + "largeone.bmp", varptr(screens(COUNTING, 0))
noise2 ( &Hff00ff, varptr(screens(COUNTING, 0)), &h800000, &hff0000 )
bsave "graphics/" + "large1.bsv", varptr(screens(COUNTING, 0)), (320*240*4+4)

bload "graphics/" + "largetwo.bmp", varptr(screens(COUNTING, 0))
noise2 ( &Hff00ff, varptr(screens(COUNTING, 0)), &h800000, &hff0000  )
bsave "graphics/" + "large2.bsv", varptr(screens(COUNTING, 0)), (320*240*4+4)

bload "graphics/" + "largethree.bmp", varptr(screens(COUNTING, 0))
noise2 ( &Hff00ff, varptr(screens(COUNTING, 0)), &h800000, &hff0000  )
bsave "graphics/" + "large3.bsv", varptr(screens(COUNTING, 0)), (320*240*4+4)

bload "graphics/" + "cursor.bmp", varptr(cursor(0))

bload "graphics/" + "font.bmp", varptr(fontA(0))
for i = 0 to 93
	font(i+33, 0) = (9 shl 16) or (7*8)
	font2(i+33, 0) = (9 shl 16) or (7*8)
	put varptr(font(i+33, 0)), (-i*7, 0), fontA(0), pset
	put varptr(font2(i+33, 0)), (-i*7, 0), fontA(0), pset
	noise2(&HFF00FF, varptr(font2(i+33, 0)), rgb(160, 160, 160), rgb(255, 255, 255))
next i

starImg(0) = imageCreate(50, 50)
starImg(1) = imageCreate(100, 100)
starImg(2) = imageCreate(100, 100)
starImg(3) = imageCreate(100, 100)
starImg(4) = imageCreate(130, 130)

bload "graphics/" + "star50-1.bsv", starImg(0)
bload "graphics/" + "star100-1.bsv", starImg(1)
bload "graphics/" + "star100-2.bsv", starImg(2)
bload "graphics/" + "star100-3.bsv", starImg(3)
bload "graphics/" + "star130-1.bsv", starImg(4)

lmousex = xres/2
lmousey = yres/4

mainMenu:

f = freefile
open "highscores.dat" for binary as #f
	if (lof(f) = 0) then
		for i = 1 to levels
			bestTime(i) = 0
		next i
		put #f, , bestTime()
	else
		get #f, , bestTime()
	end if
close #f

if len(rtrim$(ltrim$(command$))) and story = 1 then

	filename = rtrim$(ltrim$(command$))

	if len(filename) > len("level") then
		if lcase$(mid$(filename, 1, 5)) = "level" then
			levelNo = val(mid$(filename, 6, len(filename)-5))
			if levelNo > 2 and bestTime(levelNo-1) = 0 then
				goto exitGame
			end if
		end if
	end if
	
	story = 0
	goto playGame

else

	if len(rtrim$(ltrim$(command$))) then goto exitGame

end if

initStars

if story then

	playMp3 ( "music/Theme.mp3", 0 )

	for i = 1 to 4

		dim as double intime = 1, outtime = 1

		if i = 1 then intime = 3
		if i = 4 then outtime = 0.6

		bload "graphics/" + "intro" + ltrim$(str$(i)) + ".bmp", varptr(screens(COUNTING, 0))
		
		timeStart = timer
		do
			t2 = (timer - timeStart) / intime
			if t2 > 1.0 then t2 = 1.0
			t2 = t2*t2

			clsBb
			put backBuffer, (1,0), screens(COUNTING, 0), alpha, 63
			put backBuffer, (-1,0), screens(COUNTING, 0), alpha, 63
			put backBuffer, (0,1), screens(COUNTING, 0), alpha, 63
			put backBuffer, (0,-1), screens(COUNTING, 0), alpha, 63
			put backBuffer, (0,0), screens(COUNTING, 0), custom, @brightBlend2
			put backBuffer, (0,0), black, alpha, (1.0-t2)*255
			flipBb

			if cancelKey then goto endStory
		loop until t2 >= 1.0

		timeStart = timer
		while (timer-timeStart) < ((15.8/4)-(intime+outtime))
			if cancelKey then goto endStory
		wend

		if i = 4 then line black, (0, 0)-(xres-1, yres-1), &hffffff, bf

		timeStart = timer
		do
			t2 = (timer - timeStart) / outtime
			if t2 > 1.0 then t2 = 1.0

			clsBb
			put backBuffer, (1,0), screens(COUNTING, 0), alpha, 63
			put backBuffer, (-1,0), screens(COUNTING, 0), alpha, 63
			put backBuffer, (0,1), screens(COUNTING, 0), alpha, 63
			put backBuffer, (0,-1), screens(COUNTING, 0), alpha, 63
			put backBuffer, (0,0), screens(COUNTING, 0), custom, @brightBlend2
			put backBuffer, (0,0), black, alpha, t2*255
			flipBb

			if cancelKey then goto endStory
		loop until t2 >= 1.0

	next i

	dim as string lines(400)

	f = freefile
	open "story.txt" for input as #f

		for i = 0 to 399

			line input #f, lines(i)

			if lines(i) = "EOF" then exit for

			lines(i) = ltrim$(rtrim$(lines(i)))
	
		next i
	
	close #f

	bload "graphics/" + "antiman.bmp", varptr(screens(COUNTING, 0))
	line fimg, (0, 0)-(xres, yres), 0, BF
	line cBuffer, (0, 0)-(xres, yres), &hff00ff, BF

	storyFade = 0

	dim as double ypos = 290

	timeStart = timer

	do

		frameStart = timer

		t1 = frameStart - timeStart

		clsBb

		ypos = 290 - (frameStart - timeStart)*12.5

		dim as double sypos = ypos
		if ypos > 241 then ypos = 241

		burn2 (cBuffer+1), (fimg+1), &hFFFFFF
		put cBuffer, (5, 0), screens(COUNTING, 0), pset

		put backBuffer, (0, 0 + (ypos - 240)), fimg, pset
		renderStars(0, -((sypos*3)/tilesize)*8)

		put backBuffer, (5, 0 + (ypos - 240)), screens(COUNTING, 0), custom, @brightBlend2


		if t1 < 0.6 then

			put backBuffer, (0,0), black, alpha, (1.0-(t1/0.6))*255

		end if

		for j = 0 to i-1

			dim as integer xpos = 160 - fix(len(lines(j))*7/2)

			f = 20 * sin(timer*(j/5)) + 20

			printt( lines(j), xpos, fix(ypos) + j*9, rgb(155, 155, 155+f))
		
		next j

		getmouse mousex, mousey, , buttons

		if storyFade = 0 and (cancelKey or (ypos + (i-1)*9) < -10) then

			line black, (0, 0)-(xres-1, yres-1), 0, bf
			storyFade = timer

		end if

		if storyFade > 0 then

			t2 = (timer - storyFade) / 1.0

			if (t2 > 1) then
				t2 = 1
			end if

			setMusicVol options.musicVol * (1.0-t2)

			put backBuffer, (0, 0), black, alpha, t2*255

			if t2 = 1 then exit do

		end if

		flipBb
		screensync

		while (timer - frameStart) < (1/fps): wend

		ypos = sypos
		
	loop

end if

endstory:

do
	getmouse mousex, mousey, , buttons

loop until buttons = 0

story = 0

options.firstRun = 0

'' menu

bload "graphics/" + "antiman.bmp", varptr(screens(COUNTING, 0))

timeStart = timer
menuFade = timeStart

for i = 1 to levels

	f = freefile
	open "levels/" + "level" + ltrim$(rtrim$(str$(i))) + ".lev" for binary as #f
		get #f, , levelInfo(i)
	close #f

next i

mousex = lmousex
mousey = lmousey
setmouse (lmousex), (lmousey)
'mousex = xres/2: mousey = yres/2

line fimg, (0, 0)-(xres, yres), 0, BF
line cBuffer, (0, 0)-(xres, yres), &hff00ff, BF
line black, (0, 0)-(xres, yres), 0, BF

frameCount = 0
help = 0
optMenu = 0
lbuttonhold = 1

while len(inkey$): wend

setMusicVol 0
playMp3 "music/Menu.mp3", 1
lastSelected = -1

do

	dim selected = 0
	dim as integer curlevelmax = 0
	dim ky as string = inkey$

	frameStart = timer

	if menuFade = 0 then
		getmouse mousex, mousey, , buttons
	end if

	clsBb

	if help = 0 then

		frameCount += 1

		if frameCount = 2 then

			burn2 (cBuffer+1), (fimg+1), &hFFFFFF
			put cBuffer, (0, 0), screens(COUNTING, 0), pset
			frameCount = 0

		end if

		put backBuffer, (0, -70), fimg

	end if

	renderStars(0, ((timer-timeStart)*50*10)/tilesize)

	if help = 0 then
		put backBuffer, (0, -70), screens(COUNTING, 0), custom, @brightBlend2
	elseif help = 1 then
		put backBuffer, (0, 1), screens(COUNTING, 0), custom, @brightBlend2
	elseif help = 2 then
		put backBuffer, (0, 1), screens(COUNTING, 0), custom, @brightBlend2
	end if
	
	dim as string tx = "X"

	if help > 0 then tx = "Back"
	if optMenu > 0 then tx = "Back"

	if mousex >= 5 and mousex <= (5 + (7*len(tx))+2) and _
	   mousey >= (5) and mousey <= (6 + 9) then
		selected = -1
		line backBuffer, (4, 4)-step(7*len(tx)+1+2, 8+2), rgb(91, 91, 91), b
		printt ( tx, 6, 6, rgb(230, 230, 230))
	else
		printt ( tx, 6, 6, rgb(160, 160, 160))
	end if

	if optMenu > 0 then
		printt ( "Options:", cenof("Options:"), 85, rgb(127, 127, 255))

		for i = 1 to 4

			dim as integer scale
			dim as string optName

			dim as string itms(20)
			dim as integer yp = 89 +12 + 8 + (i-1) * 32
			dim as integer curOptSel

			if i = 1 or i = 2 then
				scale = 7
				itms(0) = "Off": itms(1) = "1": itms(2) = "2": itms(3) = "3": itms(4) = "4"
				itms(5) = "5": : itms(6) = "Max"
			end if
			if i = 3 then
				scale = 3
				itms(0) = "Low": itms(1) = "Med": itms(2) = "High"
			end if
			if i = 4 then
				scale = 2
				itms(0) = "Off": itms(1) = "On"
			end if
			'if i = 4 then
			'	scale = 0
			'end if

			if i = 1 then optName = "Sound Volume"
			if i = 2 then optName = "Music Volume"
			if i = 3 then optName = "Graphics Detial"
			if i = 4 then optName = "Video Compatability"
			'if i = 4 then optName = "Save"

			if i = 1 then
				if options.soundVol = -1 then
					curOptSel = 0
				elseif options.soundVol = 100 then
					curOptSel = 6
				else
					curOptSel = fix(options.soundVol * (6/100)) + 1
				end if
			end if

			if i = 2 then
				if options.musicVol = -1 then
					curOptSel = 0
				else
					curOptSel = fix(options.musicVol * (6/100)) + 1
				end if
			end if

			if i = 3 then curOptSel = options.graphicsDetail

			if i = 4 then curOptSel = newCompMode

			dim as integer xp = cenof(("+"+space$(14)+"+"))

			printt ( optName, xp, yp, rgb(91, 91, 91) )

			yp = yp + 10

			if ky = (chr$(255) + "P") then

				if lastSelected <= (-i * 10) and lastSelected > (-(i+1) * 10) and i < 3 then
					mousex = xp + 7
					mousey += 32
					setmouse mousex, mousey
					selected = 0
					ky = ""
				end if
			
			end if

			if ky = (chr$(255) + "H") then

				if lastSelected <= (-i * 10) and lastselected > (-(i+1) * 10) and i > 1 then
					mousex = xp + 7
					mousey -= 32
					setmouse mousex, mousey
					selected = 0				
					ky = ""
				end if
			
			end if

			for j = 0 to scale-1

				dim as integer clr = rgb(0, 0, 160)
				dim as integer thisSel = (-i * 10) - j

				if lastSelected = thisSel then

					if ky = (chr$(255) + "M") then
						if j < (scale-1) then
							mousex = xp + len(itms(j)) * 7 + 7 + (len(itms(j+1)) * 7 / 2) 
							setmouse mousex, mousey
							selected = 0
							ky = ""
						end if
					end if

					if ky = (chr$(255) + "K") then
						if j > 0 then
							mousex = xp - len(itms(j-1)) * 7 - 7 + (len(itms(j-1)) * 7/2)
							setmouse mousex, mousey
							selected = 0
							ky = ""
						end if
					end if

				end if

				if curOptSel = j then clr = rgb(91, 91, 0)

				if mousex >= xp and mousex <= (xp + len(itms(j))*7) and mousey >= yp and mousey <= (yp + 11) then

					selected = thisSel

					if curOptSel = j then clr = rgb(192, 192, 0) else clr = rgb(0, 0, 255)
					
				end if

				printt ( itms(j), xp, yp, clr )

				if selected = thisSel then

					line backBuffer, (xp-2, yp-2)-step(len(itms(j))*7+2, 8+2), rgb(91, 91, 91), b
				
				end if

				xp += len(itms(j))*7 + 7
			
			next j
	
		next i

	end if

	dim xpos = cenof("select a level:")

	if help = 0 and optMenu = 0 then

		printt ( "Select a Level:", xpos, 85, rgb(127, 127, 255))
		'printtb ( cBuffer, "Select a Level:", xpos, 85, rgb(255, 255, 255))

		for i = 1 to levels

			dim as string txt = ltrim$(rtrim$(levelInfo(i).lname))
			dim as integer clr = rgb(91, 91, 91)

			if i <= 2 or bestTime(i-1) > 0 then
				curlevelmax = i
				if mousex >= CENOF(txt) and mousex <= (CENOF(txt) + len(txt)*7) and _
				   mousey >= (89 + i * 12) and mousey <= (89+11 + i*12) then
					selected = i
					levelNo = i
					clr = rgb(192, 192, 192)
					'printtb ( cBuffer, txt, CENOF(txt), 90 + i * 12, &hffffff)
				end if			
			else
				clr = 0
			end if

			if newHighScore = i then clr = rgb(255, 255, 0)

			if selected = i then
				if newHighScore = i then
					line backBuffer, (CENOF(txt)-2, 90+i*12-2)-step(len(txt)*7+2, 8+2), rgb(141, 141, 141), b
				else
					line backBuffer, (CENOF(txt)-2, 90+i*12-2)-step(len(txt)*7+2, 8+2), rgb(91, 91, 91), b
				end if
			end if

			if clr <> 0 then printt ( txt, CENOF(txt), 90 + i * 12, clr)
	
		next i

	end if

	if selected <= 0 then

		if optMenu = 0 then
		
			if mousex >= (319-5 - 4*7) and mousex <= (319 - 5) and _
			   mousey >= (5) and mousey <= (6 + 9) then
				selected = -2
				line backBuffer, (319-1-5-4*7-1, 5-1)-step(8+3*7+2, 8+2), rgb(91, 91, 91), b
				if help > 0 then
					printt ( "Next", 319-5-4*7, 6, rgb(230, 230, 230))
				else
					printt ( "Help", 319-5-4*7, 6, rgb(230, 230, 230))
				end if
			else
				'line (319-1-5-4*7, 5)-step(8+3*7, 8), rgb(61, 61, 61), b
				if help > 0 then
					printt ( "Next", 319-5-4*7, 6, rgb(160, 160, 160))
				else
					printt ( "Help", 319-5-4*7, 6, rgb(160, 160, 160))
				end if
			end if
			
		end if

		if optMenu = 0 and help = 0 then

			if mousex >= (319-5 - 7*7) and mousex <= (319 - 5) and _
			   mousey >= (5+12) and mousey <= (6 + 9+12) then
				selected = -3
				line backBuffer, (319-1-5-7*7-1, 5-1+12)-step(8+6*7+2, 8+2), rgb(91, 91, 91), b
				printt ( "Options", 319-5-7*7, 6+12, rgb(230, 230, 230))
			else
				'line (319-1-5-4*7, 5)-step(8+3*7, 8), rgb(61, 61, 61), b
				printt ( "Options", 319-5-7*7, 6+12, rgb(160, 160, 160))
			end if

		end if

	end if

	if selected > 0 then

		if ky = (chr$(255) + "H") then

			if selected > 1 then setmouse xres/2, mousey-12

		end if

		if ky = (chr$(255) + "P") then

			if selected < curlevelmax then setmouse xres/2, mousey+12

		end if

		dim as string txt

		if bestTime(selected) = 0 then
			txt = "Beat it!"
		else
			if newHighScore = selected then
				txt = "New High Score!"
			else
				txt = "Best Time"
			end if
		end if

		printt( txt, xres-4-len(txt)*7, 4, rgb(255, 255, 0) )

		if bestTime(selected) > 0 then

			txt = makeTime(bestTime(selected))
	
			printt( txt, xres-4-len(txt)*7, 4+10, rgb(255, 255, 0) )

		end if
	
	elseif ky = (chr$(255) + "P") or ky = (chr$(255) + "H") or ky = (chr$(255) + "M") or ky = (chr$(255) + "K") then

		if selected = 0 then
			if optMenu then
				setmouse 89 +12 + 8+5, yres/2
			else
				setmouse xres/2, yres/2
			end if
		end if

	end if

	if optMenu > 0 then

		if selected <> 0 and (((buttons and 1) <> 0 and lbuttonhold = 0) or (ky = chr$(13))) then

			if selected <= -10 and selected > -20 then

				dim as integer olev = abs(selected + 10)

				if olev = 0 then options.soundVol = -1
				if olev > 0 then options.soundVol = fix(100/6) * olev

				setSoundVol ( options.soundVol )
			
			end if

			if selected <= -20 and selected > -30 then

				dim as integer olev = abs(selected + 20)

				if olev = 0 then options.musicVol = -1
				if olev > 0 then options.musicVol = fix(100/6) * olev
			
				setMusicVol ( options.musicVol )
			
			end if

			if selected <= -30 and selected > -40 then

				options.graphicsDetail = abs(selected + 30)
				initStars
			
			end if

			if selected <= -40 and selected > -50 then
				newCompMode = abs(selected + 40)
			end if

		end if
	
	end if

	if selected <> 0 and (((buttons and 1) <> 0 and lbuttonhold = 0) or (ky = chr$(13))) then
		playSoundEx(dingSample, 35000, 35, 0)
		lastSelected = -100
	end if

	if optMenu = 0 and menuFade = 0 and (((buttons and 1) <> 0 and selected = -3 and lbuttonhold = 0) or multikey(SC_F2)) then
	
		optMenu = 1
		selected = 0
		
	end if

	if optMenu > 0 and menuFade = 0 and (((buttons and 1) <> 0 and selected = -1 and lbuttonhold = 0)) then
		optMenu = 0
		selected = 0
	end if

	if help = 0 and menuFade = 0 and (((buttons and 1) <> 0 and selected = -2 and lbuttonhold = 0) or multikey(SC_F1)) then
	
		help = 1
		selected = 0
		bload "graphics/" + "help1.bmp", varptr(screens(COUNTING, 0))
	
	end if

	if help > 0 and menuFade = 0 and (((buttons and 1) <> 0 and selected = -2 and lbuttonhold = 0) or ky = chr$(255) + "M") then
	
		help += 1
		if help > 2 then help = 0

		if help = 0 then bload "graphics/" + "antiman.bmp", varptr(screens(COUNTING, 0))
		if help = 1 then bload "graphics/" + "help1.bmp", varptr(screens(COUNTING, 0))
		if help = 2 then bload "graphics/" + "help2.bmp", varptr(screens(COUNTING, 0))
		selected = 0
	
	end if

	if help > 0 and menuFade = 0 and (((buttons and 1) <> 0 and selected = -1 and lbuttonhold = 0) or ky = chr$(255) + "K") then
	
		help -= 1

		if help = 0 then bload "graphics/" + "antiman.bmp", varptr(screens(COUNTING, 0))
		if help = 1 then bload "graphics/" + "help1.bmp", varptr(screens(COUNTING, 0))
		if help = 2 then bload "graphics/" + "help2.bmp", varptr(screens(COUNTING, 0))
		selected = 0

	end if

	if menuFade = 0 and ((buttons and 1) <> 0 or multikey(SC_ENTER)) and selected > 0 then

		filename = "level" + ltrim$(rtrim$(str$(selected)))
		menuFade = timer
	
	end if

	if menuFade > 0 then

		t1 = (timer - menuFade) / 1.0

		if t1 > 1.0 then t1 = 1.0

		if menuFade = timeStart then

			put backBuffer, (0, 0), black, alpha, (1.0-t1)*255
			setMusicVol options.musicVol*t1

		else

			if t1 = 1.0 then
				lmousex = mousex
				lmousey = mousey
				stopMp3
				goto playGame
			end if

			put backBuffer, (0, 0), black, alpha, t1*255
			setMusicVol options.musicVol*(1.0-t1)

		end if

		if t1 = 1.0 then menuFade = 0

	else

		dim as uinteger clr = rgb(255, 255, 255)
		'put cBuffer, (mousex, mousey), cursor(0), custom, @myblit, @clr
		put backBuffer, (mousex, mousey), cursor(0), alpha, 200
	
	end if

	while (timer - frameStart) < (1/fps): wend

	flipBb

	if ((((buttons and 1) <> 0 and selected = -1) or ky = chr$(27)) and help = 0 and optMenu = 0) and lbuttonhold = 0 then exit do

	if ky = chr$(27) and help > 0 then
		help = 0
		bload "graphics/" + "antiman.bmp", varptr(screens(COUNTING, 0))
	end if

	if ky = chr$(27) and optMenu > 0 then
		optMenu = 0
	end if

	if selected <> lastSelected and selected <> 0 then
		playSoundEx(clickSample, 11000, 10, 0)
	end if

	if (buttons and 1) then lbuttonhold = 1 else lbuttonhold = 0

	lastSelected = selected

loop

goto exitGame

playGame:

setMusicVol 0
'setSoundVol 0

initStars

if levelNo = 11 then

	playMp3 "music/Final.mp3", 1
	options.lastMusic = ""

else

	i = 0
	theDir = dir("music/random/*")
	do
		if theDir = "" then exit do
		i += 1
		theDir = dir()
	loop

	if i > 1 then

		j = 0
		theDir = dir("music/random/*")
		do
			if theDir = "" then exit do
			j += 1
			if theDir <> options.lastMusic then
				if fix(rnd * i * 0.99) = 0 then
					options.lastMusic = theDir
					playMp3 "music/random/" + theDir, 1
					exit do
				end if
			end if

			if j = i then
				theDir = dir("music/random/*")
				j = 0
			elseif j > 0 then
				theDir = dir()
			end if
		loop

	elseif i > 0 then

		theDir = dir("music/random/*")
		options.lastMusic = theDir
		playMp3 "music/random/" + theDir, 1

	else

	end if

end if

player = bplayer
newHighScore = 0

filename = "levels/" + filename

f = freefile
open filename + ".amm" for binary as #f

if lof(f) = 0 then
	close #f
	kill filename + ".amm"
	print "Error: File empty or does not exist"
	end
end if

get #f, , tiles
redim tile(1 to tiles, 0 to 32*32+4) as integer
get #f, , tile()
get #f, , map()
get #f, , player.x
get #f, , player.y
redim mines(0 to maxmines-1) as mineT
if not eof(f) then
	get #f, , minecount
	get #f, , mines()
end if
if not eof(f) then
	get #f, , dtilecount
	if dtilecount > 0 then
		redim dtile(1 to dtilecount) as dtileT
		redim dtileX(1 to dtilecount) as dtileXT
		get #f, , dtile()
	else
		redim dtile(1 to 1) as dtileT
		redim dtileX(1 to 1) as dtileXT
	end if
end if

player.x += 1: player.y += 1
for i = 1 to maxmines-1
	if mines(i).inuse then mines(i).x += 1: mines(i).y += 1
	movemines(i).inuse = 0
next i

close #f

f = freefile
open filename + ".lev" for binary as #f

if lof(f) = 0 then
	close #f
	kill filename + ".lev"
	print "No level header file (.lev), assuming default settings"
else
	get #f, , level
	dtileinterval = level.ldtinterval
	dtileforce = level.ldtforce
	levelTime = level.ltime
	gravityFactor = level.lgravfactor
	close #f
end if


for i = 1 to maxmines-1
	if mines(i).inuse then

		k = fix(mines(i).x)
		f = fix(mines(i).y)

		for j = 0 to 9
			if minemap(k, f, j) = 0 then

				minemap(k, f, j) = i
				exit for
			
			end if
		next j
	
	end if
next i

player.shield = 100

lastLargeNumber = -1

completeFlag = 0
justStart = 1

timeStart = timer
levelTime += 3
fadeOut = 0

lastLargeNumber = -1

do

	dim as integer warningFlag = 0

	framestart = timer

	' update player
	player.xa = 0: player.ya = 0

	if fix(timer - timeStart) >= 3 then
	
		if (frameStart - lastCollision) > 0.125 then
	
			if multikey(SC_RIGHT) then player.xa += demineraccel: justStart = 0
			if multikey(SC_LEFT) then player.xa -= demineraccel: justStart = 0
			if multikey(SC_UP) then player.ya -= demineraccel: justStart = 0
			if multikey(SC_DOWN) then player.ya += demineraccel: justStart = 0
			if multikey(SC_SPACE) then
				player.lastboost = framestart: player.xa *= 1.8: player.ya *= 1.8
			end if

			if multikey(SC_CONTROL) <> 0 and multikey(SC_H) <> 0 then player.shield = 100

			if multikey(SC_LSHIFT) then
				'player.xa = 0: player.ya = 0
				player.xa /= 1.25: player.ya /= 1.25
				'player.lastboost = 0
	
				if (player.xv > 0) then
					player.xv -= demineraccel/2
					if (player.xv < 0) then player.xv = 0
				elseif (player.xv < 0) then
					player.xv += demineraccel/2
					if (player.xv > 0) then player.xv = 0
				end if

				if (player.yv > 0) then
					player.yv -= demineraccel/2
					if (player.yv < 0) then player.yv = 0
				elseif (player.yv < 0) then
					player.yv += demineraccel/2
					if (player.yv > 0) then player.yv = 0
				end if

			end if

		end if

	end if

	if justStart = 1 and gravityFactor <> 0 then player.ya -= (gravity * gravityFactor)
	
	player.xv += player.xa
	player.yv += player.ya

	dim as double theAccel = sqr(player.xa*player.xa + player.ya*player.ya)

	if theAccel > 0 then

		dim as double tx1 = (timer - player.lastboost) / boosttime

		if tx1 > 1.0 then tx1 = 1.0

		tx1 = 1.0 - tx1
	
		playSoundExC ( dtileSample, 28000 + (4000 * tx1), 35, 0, 1, 1 )

	else

		stopSound 1
		
	end if

	if gravityFactor <> 0 then player.yv += (gravity * gravityFactor)
	
	if (timer - player.lastslow) > boosttime then

		player.maxv = deminermaxv
	
		if (timer - player.lastboost) > boosttime then

			player.bt = 0
			player.maxv = deminermaxv

		else

			player.bt = 1.0 - ((timer - player.lastboost) / boosttime)
			player.maxv = (player.bt * deminermaxv * 1.5) + ((1.0-player.bt) * deminermaxv)

		end if

	else

		player.bt = 1.0 - ((timer - player.lastslow) / boosttime)
		player.maxv = (player.bt * deminermaxv / 2) + ((1.0-player.bt) * deminermaxv)

	end if

	dim as double tttx = (frameStart - lastCollision) / 0.125

	if tttx <= 1.0 then

		capVel(player.xv, player.yv, tttx*player.maxv + (1.0-tttx)*dtilehitmaxv)

	else

		capVel(player.xv, player.yv, player.maxv)

	end if

	if player.shield > 0.001 then
		if player.shield < 100 then player.shield += shieldinc
	else
		warningFlag = 1
	end if

	'player.x += player.xv
	'player.y += player.yv
	
	' update camera

	' collision detection

	line cBuffer, (0, 0)-(xres-1, yres-1), 0, bf ' cls
	renderMap(camx, camy, 0)
	renderDtilesStep(camx, camy, 150)

	dim as integer resolved

	'do

	j = 0

	player.bx = player.x
	player.by = player.y
	bx = player.x
	by = player.y

	resolved = 1

	for i = 1 to 150

		player.x += player.xv/150
		player.y += player.yv/150

		toScreenPos(camx, camy, player.x, player.y, ix, iy)

		'if i = 1 or (i mod 20) = 0 or i = 150 then
		'end if
		
		if collision (ix, iy, varptr(deminer(3, 0)), xd, yd) then

			resolved = 0

			'player.x = bx
			'player.y = by

			dim as integer either = 0

			if abs(player.xv) < (1/fps) and abs(player.yv) < (1/fps) and (yd >= 7) then

				if cofx = 0 and cofy = 0 then

					player.xv = 0
					player.yv = 0
					player.x = bx
					player.y = by
					exit for

				end if

			end if

			dim as double nx, ny, nd
			dim as double px, py

			dim as double mp = 1.0, mb = 100

			if cofx <> 0 or cofy <> 0 then mb *= dtileforce

			'px = ((player.x - camx) * tilesize) + 8
			'py = ((player.y - camy) * tilesize) + 8
			'nx = xd - px
			'ny = yd - py
			nx = (xd-8)
			ny = (yd-8)
		
			xd = (xd-8) / tilesize
			yd = (yd-8) / tilesize

			if cofx = 0 and cofy = 0 and abs(player.xv) < (1.5/fps) and abs(player.yv) < (1.5/fps) and (frameStart - lastCollision) > 0.125 then
			
				dim as integer flag = 2
				dim as double lxd, lyd

				player.xv = 0
				player.yv = 0
			
				while flag > 1

					if collision (ix, iy, varptr(deminer(3, 0)), xd, yd) = 0 then
						flag -= 1
					else
						lxd = xd
						lyd = yd
					end if

					dim as double mag1 = sqr((lxd-8)^2 + (lyd-8)^2)
		
					xd = ((lxd-8)/mag1) / tilesize
					yd = ((lyd-8)/mag1) / tilesize

					player.x -= xd*2
					player.y -= yd*2

					toScreenPos(camx, camy, player.x, player.y, ix, iy)

				wend

			else

				nd = sqr(nx*nx + ny*ny)
				nx /= nd
				ny /= nd

				dim as double tx, ty
				tx = -ny
				ty = nx

				dim as double vait = player.xv*tx + player.yv*ty
				dim as double vain = player.xv*nx + player.yv*ny
				dim as double vbin = cofx*nx + cofy*ny
				dim as double cofE = 0.5

				dim as double vafn = (mb*vbin*(1+cofE) + vain*(mp - (cofE*mb))) / (mp + mb)

				dim as double rx, ry

				rx = vafn*nx + vait*tx
				ry = vafn*ny + vait*ty
	
				dim as double dmg = 0

				dmg = sqr((player.xv - rx)^2 + (player.yv - ry)^2) * shielddamage

				playSoundEx ( bounceSample, 55100, 35+50*(dmg/30), 0 )

				'if dmg < 17.5 then dmg = 0

				player.xv = rx
				player.yv = ry

				if player.shield > 0.001 then
					player.shield -= dmg
					if player.shield < 0 then 
						warningStart = timer
						player.shield = 0.00005
					end if
				elseif dmg > 10 then
					if (timer-warningStart) > warningTime then
						player.shield = 0
						player.x = player.bx
						player.y = player.by
						resolved = 1
					end if
				end if

				if collCount > 1 then player.shield = 0

				if cofx <> 0 or cofy <> 0 then lastCollision = frameStart

				if levelNo = 10 and (cofx <> 0 or cofy <> 0) then player.shield = 0

				if collCount = 1 and player.shield > 0 then

					dim as double icofx = cofx, icofy = cofy
		
					i = 0
					do
						player.x -= xd*0.25
						player.y -= yd*0.25
						dim as double lxd = xd, lyd = yd
						toScreenPos(camx, camy, player.x, player.y, ix, iy)
						if collision(ix, iy, varptr(deminer(3, 0)), xd, yd) = 0 then
							player.x -= lxd*0.45
							player.y -= lyd*0.45
							exit do
						end if
						if icofx <> cofx or icofy <> cofy then
							player.shield = 0
							exit do
						end if
						xd = (xd-8) / tilesize
						yd = (yd-8) / tilesize
						if collCount > 1 then
							player.shield = 0
							exit do
						end if
						i += 1
					loop until i > 20

				end if

			end if	
	
			j = 1

			exit for

		end if
		
		bx = player.x
		by = player.y
		
	next i

	if j = 0 then

		player.x = player.bx + player.xv
		player.y = player.by + player.yv
	'else
	end if

	'loop until resolved

	i = mineCollision(player.x+8/tilesize, player.y+8/tilesize, 8)

	if i > 0 then

		mines(i).inuse = 0
		movemines(i).inuse = 1
		movemines(i).x = mines(i).x
		movemines(i).y = mines(i).y
		movemines(i).st = timer

	end if

	if player.shield <= 0 then
		fadeout = 1
		fadetime = 1.75
		player.shield = 0
	end if

	cameraFocus ( camx, camy, player.x, player.y )
	toScreenPos(camx, camy, player.x, player.y, ix, iy)

	player.px(0) = player.x
	player.py(0) = player.y

	for i = 9 to 1 step -1
	
		player.px(i) = player.px(i-1)
		player.py(i) = player.py(i-1)
	
	next i

	' render
	clsBb

	renderStars(camx, camy)
	renderMines(camx, camy)
	renderDtiles(camx, camy, 1)
	renderMap(camx, camy, 1)

	for i = 1 to maxmines-1

		if movemines(i).inuse then

			t1 = (timer - movemines(i).st) / minemovetime

			if t1 < 1 then

				xd = player.x+8/tilesize
				yd = player.y+8/tilesize

				xd = xd - movemines(i).x
				yd = yd - movemines(i).y
				dist = sqr(xd*xd+yd*yd)

				xd /= dist
				yd /= dist

				dim as double nxd = -xd, nyd = -yd

				xd = movemines(i).x + xd * dist * t1
				yd = movemines(i).y + yd * dist * t1

				t1 = 1.0 - t1

				toScreenPos(camx, camy, xd, yd, ix2, iy2)

				put backBuffer, (ix2-6-1, iy2-6), antimine(0, 0), alpha, (fix(8*255*t1) mod 256) / 2
				put backBuffer, (ix2-6+1, iy2-6), antimine(0, 0), alpha, (fix(8*255*t1) mod 256) / 2
				put backBuffer, (ix2-6, iy2-6-1), antimine(0, 0), alpha, (fix(8*255*t1) mod 256) / 2
				put backBuffer, (ix2-6, iy2-6+1), antimine(0, 0), alpha, (fix(8*255*t1) mod 256) / 2
				put backBuffer, (ix2-6, iy2-6), antimine(0, 0), alpha, fix(8*255*t1) mod 256
				put cBuffer, (ix2-6, iy2-6), antimine(0, 0), trans

				'for j = 1 to 6
				'	lightningRay ( camx, camy, player.x+8/tilesize, player.y+8/tilesize, _
				'				   (nxd), (nyd), 1.0, fix(rnd*(3-0.00001)), 10)
				'next j

			else

				player.lastabsorb = timer
				player.shield += mineshieldinc
				if player.shield > 100 then player.shield = 100
				movemines(i).inuse = 0

				playSoundEx ( mineSample, -1, 40, 0 )

			end if
		
		end if
	
	next i

	for i = 1 to ubound(dtile)

		if dtile(i).inuse <> 0 and dtileX(i).onscreen = 1 then
			playDtile( (dtileX(i).x-player.x), (dtileX(i).y-player.y), dtileX(i).xv*4, dtileX(i).yv*4, i-1 )
		else
			stopDtile( i-1 )
		end if

	next i

	t1 = (timer - player.lastabsorb) / (minemovetime)

	if t1 < 1.0 then

		for i = 1 to 1
			ang = rnd*2*pi

			lightningRay ( camx, camy, player.x+8/tilesize, player.y+8/tilesize, _
						   cos(ang), sin(ang), 1.0-t1, fix(rnd*(3-0.00001)), 1.35)
		next i

	end if

	if player.xa then

		for i = 1 to 3
		
			lightningRay ( camx, camy, player.x+8/tilesize, player.y+8/tilesize, _
						   -sgn(player.xa), 0, 1.0, 0, 1+1/(1+abs(player.xa)))
						   
		next i

	end if

	if player.ya then

		for i = 1 to 3
		
			lightningRay ( camx, camy, player.x+8/tilesize, player.y+8/tilesize, _
						   0, -sgn(player.ya), 1.0, 0, 1+1/(1+abs(player.ya)))
						   
		next i

	end if

	for i = 9 to 0 step -1

		toScreenPos(camx, camy, player.px(i), player.py(i), ix2, iy2)	

		t1 = (player.shield/100)^2

		if warningFlag then t1 = sin((timer - warningStart) * pi * 4) * 0.125 + .25
		
		t2 = (i+1)^1.5

		put backBuffer, (ix2-1, iy2), deminer(0, 0), alpha, ((t1) * 63)/t2
		put backBuffer, (ix2+1, iy2), deminer(0, 0), alpha, ((t1) * 63)/t2
		put backBuffer, (ix2, iy2-1), deminer(0, 0), alpha, ((t1) * 63)/t2
		put backBuffer, (ix2, iy2+1), deminer(0, 0), alpha, ((t1) * 63)/t2
		put backBuffer, (ix2, iy2), deminer(0, 0), alpha, (255*(t1))/t2
	
'		if player.bt > 0 then
		'	put (ix2-1, iy2), deminer(1, 0), alpha, (sqr(t1/2) * 63)/t2
		'	put (ix2+1, iy2), deminer(1, 0), alpha, (sqr(t1/2) * 63)/t2
		'	put (ix2, iy2-1), deminer(1, 0), alpha, (sqr(t1/2) * 63)/t2
		'	put (ix2, iy2+1), deminer(1, 0), alpha, (sqr(t1/2) * 63)/t2
		'	put (ix2, iy2), deminer(1, 0), alpha, (sqr(t1/2) * 63)/t2
'		end if

		if i = 0 then
			put backBuffer, (ix2-1, iy2), deminer(2, 0), alpha, 127
			put backBuffer, (ix2+1, iy2), deminer(2, 0), alpha, 127
			put backBuffer, (ix2, iy2-1), deminer(2, 0), alpha, 127
			put backBuffer, (ix2, iy2+1), deminer(2, 0), alpha, 127
			put backBuffer, (ix2, iy2), deminer(2, 0), alpha, 255
		end if

'		if i > 0 and i-2 < 0 then i = 2
	
	next i

	if warningFlag then

		if fix((timer - warningStart) * 2 + 1) mod 2 then
			put backBuffer, (0, 0), screens(WARNING, 0), alpha, 255
			if warningSound = 0 then
				playSoundEx(tone2, 60000, 50, -1)
				warningSound = 1
			end if
		else
			put backBuffer, (0, 0), screens(WARNING, 0), alpha, 127
			warningSound = 0
		end if

	end if

	i = fix(timer - timeStart)

	if i > 3 then i = (3-(levelTime-fix(timer-timeStart)))

	if i >= 0 and i < 3 then

		if (fix((timer - timeStart) * 3 + 1) mod 3) then

			j = (3-(i+1))

			if lastLargeNumber <> j then

				bload "graphics/" + "large" + ltrim$(rtrim$(str$(j+1))) + ".bsv", varptr(screens(COUNTING, 0))
				playSound(clickSample)
				lastLargeNumber = j

			end if

			put backBuffer, (-1, 0), screens(COUNTING, 0), alpha, 127
			put backBuffer, (1, 0), screens(COUNTING, 0), alpha, 127
			put backBuffer, (0, -1), screens(COUNTING, 0), alpha, 127
			put backBuffer, (0, 1), screens(COUNTING, 0), alpha, 127
			put backBuffer, (0, 0), screens(COUNTING, 0), trans

		end if

	elseif i = 3 and lastLargeNumber = 0 then

		playSound(dingSample)
		lastLargeNumber = -1

	end if

'	put (ix, iy), deminer(0), trans

	line backBuffer, (5,5)-step(102, 7), rgb(192, 192, 192), b

	ix = fix(player.shield)

	for i = 0 to ix

		dim as integer red2, green2, blue2, clr2
		red2 = fix(sqr(1.0-(i/100)) * 255) shr 1
		green2 = fix(0) shr 1
		blue2 = fix(sqr(i/100) * 255) shr 1

		for j = 6 to 11
			clr2 = point(i+6, j, backBuffer)
			red = (clr2 shr 16) shr 1
			green = ((clr2 shr 8) and 255) shr 1
			blue = (clr2 and 255) shr 1
			pset backBuffer, (i+6, j), rgb(red+red2, green+green2, blue+blue2)
		next j
	
	next i
	
'	line (10,10)-step(player.shield, 10), rgb(0, 0, 255), bf

	dim as integer timeLeft = levelTime - fix(frameStart-timeStart)
	dim as double timeLeftD = levelTime - (frameStart-timeStart)

	if timeLeftD < 15 then
		if timeLeftD < 0 then
			printt(makeTime(0), 320 - 8*7 - 6, 6, rgb(255*sin(frameStart * pi * 2)^2, 0, 0))
			if completeFlag = 0 then
				fadeout = 1
				fadetime = 1.75
			end if
		else
			printt(makeTime(timeLeftD), 320 - 8*7 - 6, 6, rgb(255*sin(frameStart * pi * 2)^2, 0, 0))
		end if
	else
		if timeLeftD > level.ltime then
			printt(makeTime(level.ltime), 320 - 8*7 - 6, 6, rgb(255, 0, 0))
		else
			printt(makeTime(timeLeftD), 320 - 8*7 - 6, 6, rgb(0, 255, 0))
		end if
	end if

	dim as integer minesLeft = 0
	for i = 1 to maxmines-1
		if mines(i).inuse then minesLeft += 1
		if movemines(i).inuse then minesLeft += 1
	next i

	if minesLeft = 0 then

		if completeFlag = 0 then completeStart = timer: completeFlag = 1
	
	end if

	printn(3, minesLeft, 160 - (7*1.5), 6, rgb(255, 255, 255))

	dim as string txt
	
	if bestTime(levelNo) = 0 then 
		txt = ltrim$(rtrim$(level.lname))
	else
		txt = "Best Time: " + makeTime(bestTime(levelNo))
	end if

	dim as integer textx = xres/2 - len(txt)/2*7

	printt(txt, textx, yres-9-2, rgb(127, 127, 127))

	t1 = (timer - timeStart)

	if t1 < 1 then

		setMusicVol options.musicVol * t1
		'setSoundVol options.soundVol * t1
		put backBuffer, (0, 0), black, alpha, (1.0-t1)*255

	end if

	flipBb

	while (timer-framestart) < (1/fps): wend

	if completeFlag = 1 and (timer - completeStart) > 1 then

		lastLargeNumber = -1

		redim as integer pbg(0 to 320*240*4)

		bload "graphics/" + "complete.bmp", varptr(screens(COUNTING, 0))
		get backBuffer, (0, 0)-(xres-1, yres-1), pbg(0)

		stopAllSounds

		dim as double stt = timer, tt
		dim as double gameTime = (levelTime-3) - timeLeftD

		if bestTime(levelNo) = 0 or bestTime(levelNo) > gameTime then
			bestTime(levelNo) = gameTime
			newHighScore = levelNo
		else
			newHighScore = 0
		end if

		dim as integer changeFlag = 0

		line black, (0, 0)-step(xres, 0), rgb(255, 255, 0)

		do
			t1 = (timer - stt) / 2.0
			if t1 > 1.0 then t1 = 1.0

			clsBb
			put backBuffer, (0, 0), pbg(0), pset
			put backBuffer, (0, 0), screens(COUNTING, 0), alpha, t1*255
			put backBuffer, (1, 1), screens(COUNTING, 0), alpha, t1*92

			flipBb

			dim as double t1x = t1 * 4

			if t1x > 1.0 then

				if changeFlag = 0 then
					setMusicVol options.musicVol
					playMp3 "music/Win1.mp3", 0
					changeFlag = 1
				end if

				setMusicVol options.musicVol

			else

				setMusicVol (1.0-t1x) * options.musicVol
			
			end if

		loop until t1 = 1.0

		stt = timer
		do
	
			clsBb
			put backBuffer, (0, 0), screens(COUNTING, 0), pset
			tt = sin((timer - stt)/pi*2)
			put backBuffer, (1, 1), screens(COUNTING, 0), alpha, 92 + 91*tt

			dim as double ttx = 1

			put backBuffer, (0, yres-25-10-10-10), black, alpha, 191*ttx

			if newHighScore > 0 then
				dim as double tttx = sin(timer*pi) * 0.5 + 0.5
				printt("Your Time: " + makeTime(gameTime), cenof("Your Time: " + makeTime(gameTime)), yres-25-10, _
					rgb((220+35*tttx)*ttx, (220+35*tttx)*ttx, (50+205*tttx)*ttx))
			else
				printt("Your Time: " + makeTime(gameTime), cenof("Your Time: " + makeTime(gameTime)), yres-25-10, rgb(220*ttx, 220*ttx, 50*ttx))
			end if
			printt("Press ENTER", cenof("press enter"), yres-25, rgb(190*ttx, 190*ttx, 0))
			
			flipBb

			getmouse mousex, mousey, , buttons

		loop until multikey(SC_ENTER) or (buttons and 1)

		stt = timer

		do
			t1 = (timer - stt) / 2.0
			if t1 > 1.0 then t1 = 1.0

			dim as double ttx' = (timer - stt)

			clsBb
			put backBuffer, (0, 0), screens(COUNTING, 0), alpha, 255*(1.0 - t1)
			put backBuffer, (1, 1), screens(COUNTING, 0), alpha, (1.0-t1)*(92+(91*tt))
			'printt("Press ENTER", cenof("press enter"), yres-25, rgb(160*(1.0-t1), 160*(1.0-t1), 0*(1.0-t1)))

			'if ttx > 1 then ttx = 1

			ttx = 1.0 - t1

			put backBuffer, (0, yres-25-10-10-10), black, alpha, 191*ttx

			'if newHighScore > 0 then
			'	printt("New High Score!", cenof("New High Score!"), yres-25-10-10, rgb((255-60+30+30*sin(timer))*ttx, (255-60+30+30*sin(timer))*ttx, 63*ttx))
			'end if
			if newHighScore > 0 then
				dim as double tttx = sin(timer*pi) * 0.5 + 0.5
				printt("Your Time: " + makeTime(gameTime), cenof("Your Time: " + makeTime(gameTime)), yres-25-10, _
					rgb((220+35*tttx)*ttx, (220+35*tttx)*ttx, (50+205*tttx)*ttx))
			else
				printt("Your Time: " + makeTime(gameTime), cenof("Your Time: " + makeTime(gameTime)), yres-25-10, rgb(220*ttx, 220*ttx, 50*ttx))
			end if
			printt("Press ENTER", cenof("press enter"), yres-25, rgb(190*ttx, 190*ttx, 0))

			flipBb

		loop until t1 = 1.0

		line black, (0, 0)-step(xres+1, yres+1), 0, BF

		do
			getmouse mousex, mousey, , buttons

		loop until buttons = 0

		if levelNo = 11 and options.beatGame = 0 then

			bload "graphics/thanks.bmp", varptr(screens(COUNTING, 0))
			
			playMp3("music/BeatGame.mp3", 0)

			stt = timer
			do
				t1 = (timer - stt) / 2.0
				if t1 > 1.0 then t1 = 1.0

				clsBb
				put backBuffer, (0, 0), screens(COUNTING, 0), pset
				put backBuffer, (0, 0), black, alpha, (1.0 - t1) * 255
				flipBb
				
			loop until t1 >= 1.0

			stt = timer
			do
				t1 = (timer - stt) / 10.0
				if t1 > 1.0 then t1 = 1.0

				clsBb
				put backBuffer, (0, 0), screens(COUNTING, 0), pset
				flipBb
				
			loop until (t1 >= 1.0 and cancelKey)

			stt = timer
			do
				t1 = (timer - stt) / 2.0
				if t1 > 1.0 then t1 = 1.0

				clsBb
				put backBuffer, (0, 0), screens(COUNTING, 0), pset
				put backBuffer, (0, 0), black, alpha, t1 * 255
				flipBb
				
			loop until t1 >= 1.0

			options.beatGame = 1
		
		else

			lmousey += 12

		end if

		exit do

	end if

	if fadeout then

		dim as integer mark(0 to xres*yres-1)
		renderMap(camx, camy, 0)

		toScreenPos(camx, camy, player.x, player.y, ix, iy)
		put cBuffer, (ix, iy), deminer(0, 0), trans
		put cBuffer, (ix, iy), deminer(2, 0), trans

		scrptr = cBuffer
		k = xres*yres
		for i = 0 to k-1
			if scrptr[i] = 0 then mark(i) = 1 else mark(i) = 0
		next i

		stopAllSounds
		playSoundEx ( explosionSample, 8000, 100, 0 )
		playSoundEx ( explosionSample, 5000, 100, 0 )
		playSoundEx ( explosionSample, 10000, 100, 0 )

		for i = 1 to 0'16

			framestart = timer

			scrptr = backBuffer + 1

			k = xres*yres

			for j = 0 to k-1

				f = scrptr[j]

				red = f shr 16
				green = (f shr 8) and 255
				blue = f and 255

				if mark(j) = 0 then
					red += 16
					green += 16
					blue += 16
				elseif mark(j) = 1 then
					red += 1
					green += 1
					blue += 1
				end if

				if red > 255 then red = 255
				if green > 255 then green = 255
				if blue > 255 then blue = 255

				scrptr[j] = (red shl 16) or (green shl 8) or blue
			
			next j

			flipBb

			setMusicVol options.musicVol * (1.0 - ((timer-framestart) / (fadetime*0.4)))
			
			while (timer-framestart) < ((fadetime*0.4) / 16): wend
			
		next i

		for i = 1 to 127 step 4
			framestart = timer

			scrptr = backBuffer + 1

			k = xres*yres

			for j = 0 to k-1

				f = scrptr[j]
				red = (f shr 16) shr 2
				green = ((f shr 8) and 255) shr 2
				blue = (f and 255) shr 2

				if (j+1) < (k-1) then
					f = scrptr[j+1]

					red += (f shr 16)
					green += ((f shr 8) and 255)
					blue += (f and 255)
				else
					red += 127
				end if

				if j > 0 then
					f = scrptr[j-1]

					red += (f shr 16)
					green += ((f shr 8) and 255)
					blue += (f and 255)
				else
					red += 127
				end if

				if (j+xres) < (k-1) then
					f = scrptr[j+xres]

					red += (f shr 16)
					green += ((f shr 8) and 255)
					blue += (f and 255)
				else
					red += 127
				end if

				if j-xres > 0 then
					f = scrptr[j-xres]

					red += (f shr 16)
					green += ((f shr 8) and 255)
					blue += (f and 255)
				else
					red += 127
				end if

				red = red shr 2
				green = green shr 2
				blue = blue shr 2
					
				if i < 127 then
					red = red + 1
				else
					
					if red < 127 then red += 1
'					red = red - 16
'					if red < 0 then red = 0
				end if
				green = green - 16
				blue = blue - 16

				if red > 255 then red = 255
				if green < 0 then green = 0
				if blue < 0 then blue = 0

				scrptr[j] = (red shl 16) or (green shl 8) or blue
			
			next j
			
			flipBb
		
			while (timer-framestart) < ((fadetime*0.6) / (255/4)): wend
			
		next i

		setMusicVol options.musicVol
		playMp3 "music/Lose1.mp3", 0

		burnbmp("graphics/" + "gameover.bmp", 2.5, 1.25)

		exit do

	end if

loop until multikey(SC_ESCAPE)

while multikey(SC_ESCAPE): wend

f = freefile
kill "highscores.dat"
open "highscores.dat" for binary as #f
	put #f, , bestTime()
close #f

stopAllSounds

goto mainMenu

exitGame:

	options.compMode = newCompMode

	f = freefile
	kill "options.dat"
	options.firstRun = 0
	open "options.dat" for binary as #f
		put #f, , options
	close #f

	imagedestroy backBuffer
	imagedestroy cBuffer
	imagedestroy black
	imagedestroy fimg
	for i = 0 to ubound(starImg)
		imagedestroy starImg(i)		
	next i
	end

'''''''''''''''''''''''''''''''''''''''''''''''''

sub clsBb ( )
	line backBuffer, (0,0)-(xres-1, yres-1), 0, bf
end sub

sub flipBb ( )
	'
	' stretching for hires
	'
	if options.compMode then
           
			dim as integer y
			dim as byte ptr yscrptr, ybfr
			dim as byte ptr scrptr = screenptr
			
            for y = 0 to 239

	            screenlock
                
                yscrptr = y * 5120 + scrptr
                ybfr = y * 1280 + cast(byte ptr, backBuffer) + 4
            
                asm
                    
                    mov ecx, [yscrptr]
                    mov ebx, [ybfr]
                    mov edx, 0
                    
                stretchLoop:
                
                    mov eax, [ebx]
                    
                    mov [ecx], eax
                    add ecx, 4
                    mov [ecx], eax
                    add ecx, 2556
                    mov [ecx], eax
                    add ecx, 4
                    mov [ecx], eax
                    sub ecx, 2556
                    
                    add ebx, 4
                    add edx, 4
                    cmp edx, 1280
                    jl stretchLoop
                    
                end asm

	            screenunlock
                
            next y
            
		else
	
		put (0, 0), backBuffer, pset
		
	end if
end sub

sub renderMap ( camx as double, camy as double, aa as integer )

	dim as integer x, y, xx, yy, mi, offx, offy
	dim as uinteger clr = rgb(0, 0, 255)

	offx = tilesize - (fix(camx * tilesize) mod tilesize)
	offy = tilesize - (fix(camy * tilesize) mod tilesize)

	for x = -1 to (xres/tilesize)+2
		for y = -1 to (yres/tilesize)+2

			xx = fix(camx) + x
			yy = fix(camy) + y

			if xx >= 0 and yy >= 0 and xx < maxx and yy < maxy then

				mi = map(xx, yy)

				if mi > 0 then
					if aa then
						if options.graphicsDetail > 0 then
							put backBuffer, (x*tilesize+offx-1, y*tilesize+offy), tile(mi, 0), alpha, 127
							put backBuffer, (x*tilesize+offx+1, y*tilesize+offy), tile(mi, 0), alpha, 127
							put backBuffer, (x*tilesize+offx, y*tilesize+offy-1), tile(mi, 0), alpha, 127
							put backBuffer, (x*tilesize+offx, y*tilesize+offy+1), tile(mi, 0), alpha, 127
						end if
						put backBuffer, (x*tilesize+offx, y*tilesize+offy), tile(mi, 0), trans
					else
						put cBuffer, (x*tilesize+offx, y*tilesize+offy), tile(mi, 0), custom, @myblit, @clr
					end if

				end if

			end if
		
		next y
	next x

end sub

sub renderDtiles ( camx as double, camy as double, aa as integer )

	dim as integer x, y, frm, nfrm, mi
	dim as double dx, dy
	dim as integer i, j
	dim as double t, it
	dim as integer clr

	t = frameStart - timeStart

	for i = 1 to ubound(dtile)

		if dtile(i).inuse then

			frm = fix(t / dtileinterval) mod (dtile(i).frames)

			if (fix(dtileX(i).lt / dtileinterval) mod (dtile(i).frames)) <> frm then dtileX(i).lt = t
			
			nfrm = (frm+1) mod (dtile(i).frames+1)
			it = (fix(t*100000) mod fix(dtileinterval * 100000)) / (100000 * dtileinterval)
			
			mi = dtile(i).tile
			dim as double lt = (fix(dtileX(i).lt*100000) mod fix(dtileinterval * 100000)) / (100000 * dtileinterval)
			
			clr = rgb(i, 0, 0)

			j = 15

			dx = dtile(i).frmx(frm)+1 + (dtile(i).frmx(nfrm) - dtile(i).frmx(frm)) * (lt + (j/15)*(it-lt))
			dy = dtile(i).frmy(frm)+1 + (dtile(i).frmy(nfrm) - dtile(i).frmy(frm)) * (lt + (j/15)*(it-lt))

			toScreenPos ( camx, camy, dx, dy, x, y )

			if x > -33 and y > -33 and x < xres+1 and yres+1 then

				if aa then
					if options.graphicsDetail > 0 and j = 15 then
						put backBuffer, (x+1, y), tile(mi, 0), alpha, 127
						put backBuffer, (x-1, y), tile(mi, 0), alpha, 127
						put backBuffer, (x, y+1), tile(mi, 0), alpha, 127
						put backBuffer, (x, y-1), tile(mi, 0), alpha, 127
					end if
					if j = 15 then put backBuffer, (x, y), tile(mi, 0), trans
				else
					'dtileX(i).x = dx+0.5
					'dtileX(i).y = dy+0.5
					'dtileX(i).xv = (dtile(i).frmx(nfrm) - dtile(i).frmx(frm)) / (dtileinterval * fps)
					'dtileX(i).yv = (dtile(i).frmy(nfrm) - dtile(i).frmy(frm)) / (dtileinterval * fps)
					'put cBuffer, (x, y), tile(mi, 0), custom, @myblit, @clr
				end if

				dtileX(i).onscreen = 1

			else

				dtileX(i).onscreen = 0

			end if

		end if
	
	next i

end sub

sub renderDtilesStep ( camx as double, camy as double, j as integer )

	dim as integer x, y, frm, nfrm, mi
	dim as double dx, dy
	dim as integer i
	dim as double t, it
	dim as integer clr
	dim as integer maxv = 150

	t = frameStart - timeStart

	for i = 1 to ubound(dtile)

		if dtile(i).inuse then

			frm = fix(t / dtileinterval) mod (dtile(i).frames)

			if (fix(dtileX(i).lt / dtileinterval) mod (dtile(i).frames)) <> frm then dtileX(i).lt = t
			
			nfrm = (frm+1) mod (dtile(i).frames+1)
			it = (fix(t*100000) mod fix(dtileinterval * 100000)) / (100000 * dtileinterval)
			
			mi = dtile(i).tile
			dim as double lt = (fix(dtileX(i).lt*100000) mod fix(dtileinterval * 100000)) / (100000 * dtileinterval)
			
			clr = rgb(i, 0, 0)

			dx = dtile(i).frmx(frm)+1 + (dtile(i).frmx(nfrm) - dtile(i).frmx(frm)) * (lt + (j/maxv)*(it-lt))
			dy = dtile(i).frmy(frm)+1 + (dtile(i).frmy(nfrm) - dtile(i).frmy(frm)) * (lt + (j/maxv)*(it-lt))

			toScreenPos ( camx, camy, dx, dy, x, y )

			if x > -33 and y > -33 and x < xres+1 and yres+1 then

				dtileX(i).x = dx+0.5
				dtileX(i).y = dy+0.5
				dtileX(i).xv = (dtile(i).frmx(nfrm) - dtile(i).frmx(frm)) / (dtileinterval * fps)
				dtileX(i).yv = (dtile(i).frmy(nfrm) - dtile(i).frmy(frm)) / (dtileinterval * fps)
				put cBuffer, (x, y), tile(mi, 0), custom, @myblit, @clr

				dtileX(i).onscreen = 1

			else

				dtileX(i).onscreen = 0

			end if

		end if

		if j = maxv then dtileX(i).lt = t
	
	next i

end sub

sub renderMines ( camx as double, camy as double )

	dim as integer x, y, xx, yy, mi
	dim as integer i, mx, my

	for x = -1 to cint(xres/tilesize)+1
		for y = -1 to cint(yres/tilesize)+1

			xx = fix(camx) + x
			yy = fix(camy) + y

			if xx >= 0 and yy >= 0 and xx < maxx and yy < maxy then

				for i = 0 to 9
					mi = minemap(xx, yy, i)

					if mi > 0 and mines(mi).inuse then

						toScreenPos(camx, camy, mines(mi).x, mines(mi).y, mx, my)
						put backBuffer, (mx-6-1, my-6), antimine(0, 0), alpha, 63 + 63 * sin(timer/(pi))
						put backBuffer, (mx-6+1, my-6), antimine(0, 0), alpha, 63 + 63 * sin(timer/(pi))
						put backBuffer, (mx-6, my-6-1), antimine(0, 0), alpha, 63 + 63 * sin(timer/(pi))
						put backBuffer, (mx-6, my-6+1), antimine(0, 0), alpha, 63 + 63 * sin(timer/(pi))
						put backBuffer, (mx-6, my-6), antimine(0, 0), alpha, 255-127 + 127/2 * sin(timer/(pi))
					
					end if
				next i

			end if
		
		next y
	next x

end sub

function mineCollision ( px as double, py as double, rad as double ) as integer

	dim as integer x, y, xx, yy, mi
	dim as integer i, mx, my
	dim as double dx, dy

	for x = fix(px)-2 to fix(px)+2
		for y = fix(py)-2 to fix(py)+2

			xx = x
			yy = y

			if xx >= 0 and yy >= 0 and xx < maxx and yy < maxy then

				for i = 0 to 9
					mi = minemap(xx, yy, i)

					if mi > 0 and mines(mi).inuse then

						dx = mines(mi).x - px
						dy = mines(mi).y - py

						if (dx*dx+dy*dy) <= ((rad/tilesize + 6/tilesize)^2) then
						
							return mi

						end if
					
					end if
					
				next i

			end if
		
		next y
	next x

	return 0

end function

sub lightningRay ( camx as double, camy as double, ox as double, oy as double, _
			       dx as double, dy as double, t as double, clr as integer, disipate as double )

	dim as double x, y, d
	dim as integer i, j, k, lc
	dim as double sx, sy
	dim as integer lx(0 to 31), ly(0 to 31)
	dim as integer maxv = 350

	if options.graphicsDetail = 1 then maxv /= 1.5
	if options.graphicsDetail = 0 then maxv /= 3

	i = 0
	j = 0

	sx = fix((ox - camx) * tilesize)
	sy = fix((oy - camy) * tilesize)

	for d = 0 to maxv

		if point(sx+dx/2, sy+dx/2, cBuffer) > 0 then

			lx(i) = sx
			ly(i) = sy
			i += 1
			exit for

		end if

		sx += dx/2
		sy += dy/2

		if (j mod fix(800/32)) = 0 then

			lx(i) = sx
			ly(i) = sy
			i += 1

		end if

		j += 1

	next d

	lc = i - 1

	if lc = 0 then exit sub

	dim as integer r, g, b

	if clr = 0 then

		r = 127: g = 127: b = 255

	elseif clr = 1 then

		r = 255: g = 127: b = 127

	elseif clr = 3 then

		r = 255: g = 255: b = 255

	end if

	dim as double t2 = t

	dx = dx + rnd * 0.75 - (.75/2)
	dy = dy + rnd * 0.75 - (.75/2)

	for i = 0 to lc

		dim as double ang, dist
		ang = rnd*2*pi
		dist = rnd * sqr(256 * i)

		if i = lc then dist = 0

		lx(i) = lx(i) + rnd*-dx * dist
		ly(i) = ly(i) + rnd*-dy * dist

		if i > 0 then

			drawlazer (lx(i), ly(i), lx(i-1), ly(i-1), r, g, b, r/2, g/2, b/2, t2, t2/2)

		end if

		t2 /= disipate

	next i

end sub

sub toScreenPos ( camx as double, camy as double, obx as double, oby as double, byref sx as integer, byref sy as integer )

	dim as integer offx, offy

	offx = fix(camx*tilesize) mod tilesize
	offy = fix(camy*tilesize) mod tilesize

	sx = fix(obx * tilesize) - (fix(camx)*tilesize + offx)
	sy = fix(oby * tilesize) - (fix(camy)*tilesize + offy)
	
end sub

sub cameraFocus ( byref camx as double, byref camy as double, fx as double, fy as double )

	camx = fx - (xres/tilesize/2)
	camy = fy - (yres/tilesize/2)

	if camx < 0 then camx = 0
	if camy < 0 then camy = 0
	if camx > maxx-(xres/tilesize)-1 then camx = maxx-(xres/tilesize)-1
	if camy > maxy-(yres/tilesize)-1 then camy = maxy-(yres/tilesize)-1

end sub

sub capVel ( byref xv as double, byref yv as double, maxv as double )

	dim as double d

	d = sqr(xv*xv + yv*yv)
	
	if d > maxv then
	
		xv = xv/d * maxv
		yv = yv/d * maxv

	end if

end sub

function collision ( x as integer, y as integer, image as integer ptr, byref cx as double, byref cy as double )

	dim as integer xsz = (image[0] and &hffff) / 8, ysz = (image[0] shr 16)
	dim as integer xx, yy
	dim as integer ptr scrptr
	dim as integer transclr = rgb(255, 0, 255)
	dim as integer total = 0
	dim as integer flag = 0
	dim as integer dt, dttotal, ototal
	dim as integer used(0 to 256)
	dim as integer i, j
	dim as double d
	cx = 0
	cy = 0

	ototal = 0
	dttotal = 0

	collCount = 0

'	screenlock
	scrptr = cBuffer
	cofx = 0
	cofy = 0

	for xx = x to xx+xsz-1
		for yy = y to yy+ysz-1
			if image[1+xx-x + (yy-y)*xsz] <> transclr then
				if xx > 0 and yy > 0 and xx < xres and yy < yres then
					if scrptr[xx+yy*xres] > 0 then
						dt = scrptr[xx+yy*xres] shr 16

						if dt > 0 and used(dt) = 0 then
							cofx += (dtileX(dt).xv * dtileforce * dtileforcefactor)
							cofy += (dtileX(dt).yv * dtileforce * dtileforcefactor)
							copx += dtileX(dt).x
							copy += dtileX(dt).y
							used(dt) = 1
							dttotal += 1
						elseif dt = 0 then
							used(0) = 1
						end if
						
						cx += (xx-x)
						cy += (yy-y)
						total += 1
						flag = 1
					end if
				end if
			end if
		next yy
	next xx

	if total then 
		cx /= total
		cy /= total
	end if

	if used(0) then collCount += 1

	if dttotal then
		cofx /= dttotal
		cofy /= dttotal
		copx /= dttotal
		copy /= dttotal
		for i = 1 to ubound(dtile)
			if used(i) then
				if collCount then collCount += 1
				for j = i + 1 to ubound(dtile)
					if used(j) then
						if (dtileX(i).xv*dtileX(j).xv + dtileX(i).yv*dtileX(j).yv) < 0 then
							collCount += 2
						end if
					end if
				next j
			end if
		next i
		if collCount = 0 then collCount = 1
'		collCount += dttotali
	end if

'	screenunlock
	return flag

end function

function myblit ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger

	dim as uinteger ptr clr = cast(Integer ptr, param)

	if src <> &hff00ff then
		return *clr
	end if

	return dest

end function

function myblit2 ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger

	dim as uinteger ptr clr = cast(Integer ptr, param)

	if src <> &hff00ff then
		dim as double r = ((*clr) shr 16) / 255, g = (((*clr) shr 8) and 255) / 255, b = ((*clr) and 255) / 255
		dim as integer rs = src shr 16, gs = (src shr 8) and 255, bs = src and 255
		
		return (fix(rs*r) shl 16) or (fix(gs*g) shl 8) or fix(bs*b)
	end if

	return dest

end function

function brightBlend ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger

	'dim as uinteger ptr clr = cast(Integer ptr, param)

	'if src <> &hff00ff then
	'	return *clr
	'end if

	if src = 0 then return dest

	dim as integer rs = src shr 16, gs = (src shr 8) and 255, bs = src and 255
	dim as integer r = dest shr 16, g = (dest shr 8) and 255, b = dest and 255
	dim as integer a = ((48+rs+gs+bs) shr 2)
	dim as integer ai = 255 - a

	return rgb( ((r*ai) + (rs*a)) shr 8, ((g*ai) + (gs*a)) shr 8, ((b*ai) + (bs*a)) shr 8 )

	'dim as double a = (rs+gs+bs)/(3*255)
	'dim as double ai = 1.0 - a

	'return rgb(fix(r*ai + rs*a), fix(g*ai + gs*a), fix(b*ai + bs*a))

end function

function brightBlend2 ( src as uinteger, dest as uinteger, param as any ptr ) as uinteger

	'dim as uinteger ptr clr = cast(Integer ptr, param)

	'if src <> &hff00ff then
	'	return *clr
	'end if

	if src = 0 then return dest

	dim as integer rs = src shr 16, gs = (src shr 8) and 255, bs = src and 255
	dim as integer r = dest shr 16, g = (dest shr 8) and 255, b = dest and 255
	dim as integer a = rs
	if gs > a then a = gs
	if bs > a then a = bs
	dim as integer ai = 255 - a

	return (( ((r*ai) + (rs*a)) shr 8 ) shl 16) or ((((g*ai) + (gs*a)) shr 8) shl 8) or (((b*ai) + (bs*a)) shr 8 )

	'dim as double a = (rs+gs+bs)/(3*255)
	'dim as double ai = 1.0 - a

	'return rgb(fix(r*ai + rs*a), fix(g*ai + gs*a), fix(b*ai + bs*a))

end function

sub printtb ( buffer as integer ptr, ns as string, x as integer, y as integer, colour as uinteger )

	dim as integer i

	dim as uinteger colour2 = rgb((colour shr 16) / 3, ((colour shr 8) and 255) / 3, (colour and 255) / 3)

	for i = 1 to len(ns)

		if mid$(ns, i, 1) <> " " then
			put buffer, (x+(i-1)*7-1, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put buffer, (x+(i-1)*7+1, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put buffer, (x+(i-1)*7, y-1), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put buffer, (x+(i-1)*7, y+1), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put buffer, (x+(i-1)*7, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour
		end if
	
	next i

end sub

sub printt (ns as string, x as integer, y as integer, colour as uinteger )

	dim as integer i

	dim as uinteger colour2 = rgb((colour shr 16) / 3, ((colour shr 8) and 255) / 3, (colour and 255) / 3)

	'open "log.txt" for output as #1
'		print #1, n, ns, mid$(ns, 1, 1), len(ns)
'	close #1

	'put (x, y), numbers(0, 0), custom, @myblit, @colour
	
	for i = 1 to len(ns)

		if mid$(ns, i, 1) <> " " then
			put backBuffer, (x+(i-1)*7-1, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put backBuffer, (x+(i-1)*7+1, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put backBuffer, (x+(i-1)*7, y-1), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put backBuffer, (x+(i-1)*7, y+1), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
			put backBuffer, (x+(i-1)*7, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour
		end if
	
	next i

end sub

sub printn ( digits as integer, n as integer, x as integer, y as integer, colour as uinteger )

	dim as string ns = ltrim$(rtrim$(str$(n)))
	dim as integer i

	dim as uinteger colour2 = rgb((colour shr 16) / 3, ((colour shr 8) and 255) / 3, (colour and 255) / 3)

	while (len(ns) < digits): ns = "0" + ns: wend

	'open "log.txt" for output as #1
'		print #1, n, ns, mid$(ns, 1, 1), len(ns)
'	close #1

	'put (x, y), numbers(0, 0), custom, @myblit, @colour
	
	for i = 1 to len(ns)
	
		put backBuffer, (x+(i-1)*7-1, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
		put backBuffer, (x+(i-1)*7+1, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
		put backBuffer, (x+(i-1)*7, y-1), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
		put backBuffer, (x+(i-1)*7, y+1), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour2
		put backBuffer, (x+(i-1)*7, y), font2(asc(mid$(ns, i, 1)), 0), custom, @myblit2, @colour
	
	next i

end sub

sub initStars ( )
dim clusx(10) as double
dim clusy(10) as double
dim as integer i, j, k
dim as double ang, dist


for i = 0 to starlevels-1

	dim as integer mls = maxlargestars

	if options.graphicsDetail = 0 then mls = 0
	if options.graphicsDetail = 1 then mls = mls / 2

	for j = 0 to maxstars-1

		k = fix(rnd*2.9999)

		star(i, j).x = rnd*(xres+260) - 130
		star(i, j).y = rnd*(yres+260) - 130

		if j < mls then
			star(i, j).img = fix(rnd*4.9999)
		else
			dist = rnd*0.5
			star(i, j).img = -1
			if k = 0 then star(i, j).clr = rgb(255*dist, 255*dist, 255*dist)
			if k = 1 then star(i, j).clr = rgb(255*dist, 0, 0)
			if k = 2 then star(i, j).clr = rgb(0, 0, 255*dist)
		end if

	next j
next i
end sub

sub renderStars ( camx as double, camy as double )

	dim as integer ix, iy, clr, red, green, blue
	dim as integer i, j

	for i = starlevels-1 to 0 step -1
		for j = 0 to maxstars-1

			ix = star(i, j).x + (fix((100000-camx) * tilesize / ((i+2)^2)))
			iy = star(i, j).y + (fix((100000-camy) * tilesize / ((i+2)^2)))

			ix = (ix mod (xres + 260)) - 130
			iy = (iy mod (yres + 260)) - 130

			if star(i, j).img > -1 then
			
				put backBuffer, (ix, iy), starImg(star(i, j).img), custom, @brightBlend'alpha, 255 / (i+1)

			else

				pset backBuffer, (ix, iy), brightBlend(star(i,j).clr, point(ix, iy, backBuffer), 0)

			end if

			'clr = point(ix, iy)
			'red = (1+(clr shr 16)) * ((star(i,j).r shr 2)+1)
			'green = (1+((clr shr 8) and 255)) * ((star(i,j).g shr 2)+1)
			'blue = (1+(clr and 255)) * ((star(i,j).b shr 2)+1)
			'if red>255 then red = 255
			'if green>255 then green = 255
			'if blue>255 then blue = 255
			'pset (ix, iy), rgb(red shr 1, green shr 1, blue shr 1)

			'clr = point(ix-1, iy)
			'red = (clr shr 16) + (star(i,j).r shr 1)
			'green = ((clr shr 8) and 255) + (star(i,j).g shr 1)
			'blue = (clr and 255) + (star(i,j).b shr 1)
			'if red>255 then red = 255
			'if green>255 then green = 255
			'if blue>255 then blue = 255
			'pset (ix-1, iy), rgb(red, green, blue)

			'clr = point(ix+1, iy)
			'red = (clr shr 16) + (star(i,j).r shr 1)
			'green = ((clr shr 8) and 255) + (star(i,j).g shr 1)
			'blue = (clr and 255) + (star(i,j).b shr 1)
			'if red>255 then red = 255
			'if green>255 then green = 255
			'if blue>255 then blue = 255
			'pset (ix+1, iy), rgb(red, green, blue)

			'clr = point(ix, iy-1)
			'red = (clr shr 16) + (star(i,j).r shr 1)
			'green = ((clr shr 8) and 255) + (star(i,j).g shr 1)
			'blue = (clr and 255) + (star(i,j).b shr 1)
			'if red>255 then red = 255
			'if green>255 then green = 255
			'if blue>255 then blue = 255
			'pset (ix, iy-1), rgb(red, green, blue)
			
			'clr = point(ix, iy+1)
			'red = (clr shr 16) + (star(i,j).r shr 1)
			'green = ((clr shr 8) and 255) + (star(i,j).g shr 1)
			'blue = (clr and 255) + (star(i,j).b shr 1)
			'if red>255 then red = 255
			'if green>255 then green = 255
			'if blue>255 then blue = 255
			'pset (ix, iy+1), rgb(red, green, blue)
			
		next j
	next i

end sub

function makeTime ( t as double ) as string

	dim as integer ms = fix(t*100)
	dim as integer s = fix(ms/100) mod 60
	dim as integer m = fix(ms/6000)

	dim as string ret

	if m < 10 then ret = "0"
	ret += ltrim$(str$(m))

	ret += ":"

	if s < 10 then ret += "0"
	ret += ltrim$(str$(s))

	ret += ":"

	ms = ms mod 100

	if ms < 10 then ret += "0"
	ret += ltrim$(str$(ms))

	return ret

end function

function cancelKey ( ) as integer

	dim as integer mousex, mousey, buttons

	if options.firstRun then return 0

	getmouse mousex, mousey, , buttons

	if multikey(SC_ENTER) or (buttons and 1) then return 1
	if multikey(SC_ESCAPE) then return 2

	return 0	

end function
