#version 130
#extension GL_EXT_gpu_shader4 : enable
#define BUMP_MAPPING

uniform sampler2DArray texture;
uniform sampler2DArray normalmap;
uniform sampler2DArray specularmap;
uniform int texrange;
uniform vec3 focal;
uniform vec3 lightVector;
uniform float visibility;
uniform float daylight;

in vec3 texCoord;
in vec4 colordata;
in vec4 torchlight;

in float worldLight;
in vec3 viewVector;
in float vertdist;
in float modulation;

const float SHADOW_EXP = 1.1;
const float SHADOW_MUL = 1.2;
const float SHADOW_MAX = 0.98;

// const vec3 intervalMult  = vec3(0.019, 0.019, 0.019);
const vec3 intervalMult  = vec3(0.00048828125, 0.00048828125, 0.2); //128x128
// const float SPECULARNESS = 0.5;
const float BUMP_REACH   = 100.0;
const int   MAX_POINTS   = 50;
const float bumpofs      = 1.0/128.0;

/*
vec4 hjemmebakt_fog(vec4 color) {
	float f = min(FOG_MAXIMUM, vertdist / focal[2] * FOG_DENSITY);
	float lum = min(1.0, (0.7 + 0.2126 * color.r + 0.7152 * color.g + 0.0722 * color.b) * FOG_LUMINANCE);
	return vec4(mix(color.rgb, vec3(0.0, 0.0, lum), f), color[3]);
}
*/

void main(void) {

	if (vertdist >= focal[2]) discard; //ze end of ze world!
	vec4 color = vec4(0.0);

#ifdef BUMP_MAPPING
	if (vertdist < BUMP_REACH && texrange != 6) {

		vec3 coord = texCoord.xyz;

		//bump in rgb channel, displacement map in alpha channel
// 	if (texture2DArray(normalmap, coord.xyz).a < 1.0) {
// 		vec2 minCoord = vec2(texCoord.s - mod(texCoord.s, bumpofs), texCoord.t - mod(texCoord.t, bumpofs));
// 		vec2 maxCoord = vec2(minCoord.s + bumpofs, minCoord.t + bumpofs);
// 
// 		vec3 interval = viewVector * intervalMult; 
// 
// 		for (int loopCount = 0; texture2DArray(normalmap, coord.xyz).a < 1.0 && loopCount < MAX_POINTS; ++loopCount) {
// 			coord += interval;
// 			if (coord.s < minCoord.s) {
// 				coord.s += bumpofs;
// 			} else if (coord.s >= maxCoord.s) {
// 				coord.s -= bumpofs;
// 			}
// 			if (coord.t < minCoord.t) {
// 				coord.t += bumpofs;
// 			} else if (coord.t >= maxCoord.t) {
// 				coord.t -= bumpofs;
// 			}
// 		}
// 	}
		color = texture2DArray(texture, coord.xyz);
		
		vec3 bump = 2.0 * texture2DArray(normalmap, coord.xyz).xyz - 1.0;
		float specular = texture2DArray(specularmap, coord.xyz).r;
		float intensity = 1.0 - min(vertdist / BUMP_REACH + colordata[3], 1.0);
		float s = max(0.0, dot(reflect(lightVector.xyz, bump), viewVector));
		color.rgb += intensity * s * s * s * specular; // celestial
		// player-held light-sort-of
		s = max(0.0, dot(reflect(viewVector, bump), viewVector));
		intensity = 1.0 - min(vertdist / 16.0, 1.0);
		color.rgb += intensity * s * s * s * specular; // celestial
		
 	} else {
#endif
		//out of range: normal color get
		color = texture2DArray(texture, texCoord.xyz);
#ifdef BUMP_MAPPING
	}
#endif
	
	float df = 1.0 - pow(vertdist / focal[2], 1.85);
	if (pow(color[3], df * 2.5) < 0.1) discard; // le discard
	// if (color[3] == 0.0) discard;
	
	// upramp shadow gamma
	float shadow = 1.0 - min(SHADOW_MAX, pow(colordata[0], SHADOW_EXP) );
	
	float brightness = colordata[1]; // torchlight brightness
	
	// apply scaled lighting
	color.rgb *= min( 1.0, smoothstep(0.0, 1.0, worldLight * daylight * shadow + pow(worldLight, 1.2) * brightness ) );
	
	// mix in torchlight
	color.rgb = mix( color.rgb, torchlight.rgb, torchlight[3] * (1.0 - brightness + modulation) );
	
	color = vec4( color.rgb, visibility );
	
	float fadedist = focal[2] * 0.85;
	if (vertdist > fadedist) {
		float fstr = (vertdist - fadedist) / (focal[2] - fadedist);
		color.rgb *= 1.0 + fstr;
		color[3] *= 1.0 - fstr;
	}
	
	gl_FragColor = color;
}
